<?php
/**
 * 
 *
 * @package qa_heatmap
 */

$qahm_admin_page_config = new QAHM_Admin_Page_Config();

class QAHM_Admin_Page_Config extends QAHM_Admin_Page_Base {

	// スラッグ
	const SLUG = QAHM_NAME . '-config';

	// nonce
	const NONCE_ACTION = self::SLUG . '-nonce-action';
	const NONCE_NAME   = self::SLUG . '-nonce-name';

	private static $error_msg = array();
	private $localize_ary;

	/**
	 * コンストラクタ
	 */
	public function __construct() {
		parent::__construct();

		 // コールバック
		add_action( 'init', array( $this, 'init_wp_filesystem' ) );
		add_action( 'admin_init', array( $this, 'save_config' ) );
		add_action( 'load-qa-analytics_page_qahm-config', array( $this, 'admin_init' ) );
	}
	
	// 管理画面の初期化
	public function admin_init(){
		if( defined('DOING_AJAX') && DOING_AJAX ){
			return;
		}

		global $qahm_google_api;

		$scope   = array( 'https://www.googleapis.com/auth/webmasters.readonly' );
		if ( $this->wrap_get_option( 'gabackup_mode' ) ) {
			$scope[] = 'https://www.googleapis.com/auth/userinfo.email';
		}

		// nonceで設定したcredentialのチェック
		// 設定画面
		if ( isset( $_POST[ self::NONCE_NAME ] ) && $_POST[ self::NONCE_NAME ] && check_admin_referer( self::NONCE_ACTION, self::NONCE_NAME ) ) {
			// フォームから値が送信されていればDBに保存
			$client_id     = $this->wrap_filter_input( INPUT_POST, 'client_id' );
			$client_secret = $this->wrap_filter_input( INPUT_POST, 'client_secret' );
			$qahm_google_api->set_credentials( $client_id, $client_secret, null );

			$qahm_google_api->init(
				'Google API Integration',
				$scope,
				admin_url( 'admin.php?page=qahm-config' ),
				true
			);
		} else {
			$qahm_google_api->init(
				'Google API Integration',
				$scope,
				admin_url( 'admin.php?page=qahm-config' )
			);
		}
	}
	
	/**
	 * 初期化
	 */
	public function enqueue_scripts( $hook_suffix ) {
		if( $this->hook_suffix !== $hook_suffix ||
			! $this->is_enqueue_jquery()
		) {
			return;
		}

		global $qahm_time;
		$js_dir       = $this->get_js_dir_url();
		$data_dir     = $this->get_data_dir_url();
		$license_plan = $this->get_plan();
		$css_dir_url = $this->get_css_dir_url();

        $GOALMAX = 1;
        if ( $this->check_plan( 'goal_3' ) ) {
            $GOALMAX = 3;
		}
        if ( $this->check_plan( 'goal_10' ) ) {
            $GOALMAX = 10;
		}

		// enqueue_style
		$this->common_enqueue_style();
		wp_enqueue_script( QAHM_NAME . '-admin-page-config', $js_dir . 'admin-page-config.js', array( QAHM_NAME . '-admin-common' ), QAHM_PLUGIN_VERSION );
		wp_enqueue_style( QAHM_NAME . '-admin-page-common-css', $css_dir_url . 'admin-page-common.css', array( QAHM_NAME . '-reset' ), QAHM_PLUGIN_VERSION );
		wp_enqueue_style( QAHM_NAME . '-admin-page-config-css', $css_dir_url . 'admin-page-config.css', array( QAHM_NAME . '-reset' ), QAHM_PLUGIN_VERSION );
		if ( $license_plan ) {
//			wp_enqueue_script( QAHM_NAME . '-powerup-config', $data_dir . 'js/powerup-config.js', array( QAHM_NAME . '-admin-common' ), $qahm_time->now_unixtime() );
		}
		// enqueue script
		$this->common_enqueue_script();

        // g_clickpage の変数作成
         global $qahm_data_api;
         $goals_ary = $qahm_data_api->get_goals_array();
         $click_iframe_url = esc_url( get_home_url() );
         $measuring_page_url = array();
         $is_raw_save_all   = $this->wrap_get_option( 'is_raw_save_all' );
         if ( ! $is_raw_save_all ) {
             global $qahm_recterm;
             $measuring_pages = $qahm_recterm->get_recording_page();
             for ( $iii = 0; $iii < count($measuring_pages); $iii++ ) {
                 switch ( $measuring_pages[$iii]['type'] ) {
                     case 'home' :
                         $measuring_page_url[] = esc_url( get_home_url() );
                         break;
                     case 'p' :
                     case 'page_id' :
                         $measuring_page_url[] = esc_url( get_permalink( $measuring_pages[$iii]['ids'] ) );
                         break;
                     case 'cat' :
                     case 'tag' :
                     case 'tax' :
                         $measuring_page_url[] = esc_html( get_term_link( $measuring_pages[$iii]['ids'] ) );
                         break;
                 }
             }
         }
         $g_clickpage_ary = array();
         for ( $iii = 1; $iii <= $GOALMAX; $iii++ ) {
             $g_clickpage = isset( $goals_ary[$iii]['g_clickpage'] ) ? urldecode( $goals_ary[$iii]['g_clickpage'] ) : '';
             //set default
             if ( ! $g_clickpage ) {
                 if ( $is_raw_save_all ) {
                     $g_clickpage = $click_iframe_url;
                 } else {
                     $g_clickpage = $measuring_page_url[0];
                 }
             }
             $g_clickpage_ary[$iii] = $g_clickpage;
         }
		// inline script
		$scripts = $this->get_common_inline_script();
		$scripts['access_role']     = $this->wrap_get_option( 'access_role' );
		$scripts['is_raw_save_all'] = $this->wrap_get_option( 'is_raw_save_all' );
		$scripts['wp_time_adj'] = get_option('gmt_offset');
		$scripts['wp_lang_set'] =  get_bloginfo('language');
		$scripts['goalmax']     =  $GOALMAX;
        $scripts['g_clickpage'] = $g_clickpage_ary;
		wp_add_inline_script( QAHM_NAME . '-common', 'var ' . QAHM_NAME . ' = ' . QAHM_NAME . ' || {}; let ' . QAHM_NAME . 'Obj = ' . wp_json_encode( $scripts ) . '; ' . QAHM_NAME . ' = Object.assign( ' . QAHM_NAME . ', ' . QAHM_NAME . 'Obj );', 'before' );

		// localize
		$localize = $this->get_common_localize_script();
		$localize['data_save_month_title']     = esc_html__( 'Data Storage Period', 'qa-heatmap-analytics' );
		$localize['data_save_month_desc']      = esc_html__( 'With free plans, to the previous month. For subscribed, there is no period limit in principal.', 'qa-heatmap-analytics' );
		$localize['all_page_measure_title']    = esc_html__( 'Measure all pages', 'qa-heatmap-analytics' );
		$localize['all_page_measure_desc']     = esc_html__( 'Ignore the individual measurement settings on Heatmap management screen and measure all pages. Measurement starts automatically including when a new page is added (published).', 'qa-heatmap-analytics' );
		$localize['access_role_title']         = esc_html__( 'Access Privileges', 'qa-heatmap-analytics' );
		$localize['access_role_desc']          = esc_html__( 'Set the privileges to access the heatmap view.', 'qa-heatmap-analytics' );
		$localize['access_role_option_admin']  = esc_html__( 'Administrator only', 'qa-heatmap-analytics' );
		$localize['access_role_option_editor'] = esc_html__( 'Editor or above', 'qa-heatmap-analytics' );
        $localize['settings_saved']           = esc_attr__( 'Settings saved.', 'qa-heatmap-analytics' );
        $localize['cnv_couldnt_saved']        = esc_html__( 'Could not be saved. The value is same as before or is incorrect.', 'qa-heatmap-analytics' );
        $localize['cnv_delete_confirm']       = esc_html__( 'Are you sure to delete this goal?', 'qa-heatmap-analytics' );
        $localize['cnv_couldnt_delete']       = esc_html__( 'Could not delete. The value is incorrect.', 'qa-heatmap-analytics' );
        $localize['cnv_page_set_alert']       = esc_html__( 'You are trying to set all the pages.', 'qa-heatmap-analytics' );
        $localize['cnv_goal_numbering_alert'] = esc_html__( 'There is a skip in goal numbers. Please set goals sequentially.', 'qa-heatmap-analytics' );
        $localize['cnv_reaching_goal_notice'] = esc_attr__( 'You have the goal reached.', 'qa-heatmap-analytics' );
        $localize['cnv_saving']               = esc_attr__( 'Saving...', 'qa-heatmap-analytics' );
        $localize['cnv_load_page']            = esc_html__( 'Load the Page', 'qa-heatmap-analytics' );
        $localize['cnv_loading']              = esc_html__( 'Loading...', 'qa-heatmap-analytics' );
        $localize['nothing_page_id']          = esc_html__( 'Page that has not been visited cannot be set as a goal yet. Please wait at least one day before setting a goal.', 'qa-heatmap-analytics' );
        $localize['mail_btn_updating']        = esc_html__( 'Updating...', 'qa-heatmap-analytics' );
        $localize['mail_alert_update_failed'] = esc_html__( 'Failed updating. Please retry again.', 'qa-heatmap-analytics' );
        wp_localize_script( QAHM_NAME . '-common', QAHM_NAME . 'l10n', $localize );

		$this->localize_ary = $localize;

	}

	/**
	 * ページの表示
	 */
	public function create_html() {
		// データを取得
        global $qahm_data_api;
		global $qahm_google_api;

        $GOALMAX = 1;
        if ( $this->check_plan( 'goal_3' ) ) {
            $GOALMAX = 3;
		}
        if ( $this->check_plan( 'goal_10' ) ) {
            $GOALMAX = 10;
		}

        $siteinfo_ary = $qahm_data_api->get_siteinfo_array();



        $goals_ary = $qahm_data_api->get_goals_array();

        $sitetype_ary = [
            "general_company",
            "media_affiliate",
            "service_matching",
            "ec_ec" ,
            "general_shop" ,
            "media_owned" ,
            "service_ugc" ,
            "ec_contents" ,
            "general_ir" ,
            "media_other" ,
            "service_membershi",
            "ec_license" ,
            "general_recruit",
            "service_other" ,
            "ec_other" ,
        ];

        $lang_ja['target_user_question'] =  esc_html__( 'Which type of users do you want meet the goal?', 'qa-heatmap-analytics' );
        $lang_ja['target_individual'] =  esc_html__( 'Personal', 'qa-heatmap-analytics' );
        $lang_ja['target_corporation'] =  esc_html__( 'Corporations/Organizations', 'qa-heatmap-analytics' );


        $lang_ja['select_sitetype_question'] =  esc_html__( 'Choose a category that describes your site best.', 'qa-heatmap-analytics' );
        $lang_ja['general'] =  esc_html__( 'General', 'qa-heatmap-analytics' );
        $lang_ja['media'] =  esc_html__( 'Media', 'qa-heatmap-analytics' );
        $lang_ja['service'] =  esc_html__( 'Providing services', 'qa-heatmap-analytics' );
        $lang_ja['ec_mall'] =  esc_html__( 'EC/Mall', 'qa-heatmap-analytics' );

        $lang_ja['general_company'] = esc_html__( 'About a company/services', 'qa-heatmap-analytics' );
        $lang_ja['media_affiliate'] = esc_html__( 'Affiliate blogs/Media', 'qa-heatmap-analytics' );
        $lang_ja['service_matching'] = esc_html__( 'Matching', 'qa-heatmap-analytics' );
        $lang_ja['ec_ec'] = esc_html__( 'Product sales', 'qa-heatmap-analytics' );
        $lang_ja['general_shop'] = esc_html__( 'About stores/facilities', 'qa-heatmap-analytics' );
        $lang_ja['media_owned'] = esc_html__( 'Owned media', 'qa-heatmap-analytics' );
        $lang_ja['service_ugc'] = esc_html__( 'Posting', 'qa-heatmap-analytics' );
        $lang_ja['ec_contents'] = esc_html__( 'Online content sales', 'qa-heatmap-analytics' );
        $lang_ja['general_ir'] = esc_html__( 'IR', 'qa-heatmap-analytics' );
        $lang_ja['media_other'] = esc_html__( 'Other information dissemination', 'qa-heatmap-analytics' );
        $lang_ja['service_membershi'] = esc_html__( 'SNS/Member services', 'qa-heatmap-analytics' );
        $lang_ja['ec_license'] = esc_html__( 'License sales', 'qa-heatmap-analytics' );
        $lang_ja['general_recruit'] = esc_html__( 'Recruitment', 'qa-heatmap-analytics' );
        $lang_ja['service_other'] = esc_html__( 'Other services', 'qa-heatmap-analytics' );
        $lang_ja['ec_other'] = esc_html__( 'Other sales', 'qa-heatmap-analytics' );

        $lang_ja['membership_question'] = esc_html__( 'Does the site have "member registration"?', 'qa-heatmap-analytics' );
        $lang_ja['payment_question'] = esc_html__( 'Does the site have "payment function"?', 'qa-heatmap-analytics' );
        $lang_ja['goal_monthly_access_question'] = esc_html__( 'Enter the target number for monthly sessions.', 'qa-heatmap-analytics' );

        $lang_ja['membership_yes'] = esc_html__( 'Yes.', 'qa-heatmap-analytics' );
        $lang_ja['membership_no'] = esc_html__( 'No.', 'qa-heatmap-analytics' );
        $lang_ja['next'] = esc_html__( 'Next', 'qa-heatmap-analytics' );

        $lang_ja['payment_no'] = esc_html__( 'No.', 'qa-heatmap-analytics' );
        $lang_ja['payment_yes'] = esc_html__( 'Yes, using original system.', 'qa-heatmap-analytics' );
        $lang_ja['payment_cart'] = esc_html__( 'Using external cart system.', 'qa-heatmap-analytics' );


        $lang_ja['month_later'] = esc_html__( 'month(s) later, reaching', 'qa-heatmap-analytics' );
        $lang_ja['session_goal'] = esc_html__( 'sessions/month is the goal.', 'qa-heatmap-analytics' );

        $goal        = esc_html__( 'Goal', 'qa-heatmap-analytics' );
        $goal_title  = esc_html__( 'Goal Name', 'qa-heatmap-analytics' );
        $required    = esc_html_x( '*', 'A mark that indicates it is required item.', 'qa-heatmap-analytics' );
        $goal_number = esc_html__( 'Completions Target in a Month', 'qa-heatmap-analytics' );
        $num_scale   = esc_html__( 'completion(s)', 'qa-heatmap-analytics' );
        $goal_value  = esc_html__( 'Goal Value (ave. monetary amount per goal)', 'qa-heatmap-analytics' );
        $val_scale   = esc_html_x( 'dollar(s)', 'Please put your currency. (This is only for a goal criterion.)', 'qa-heatmap-analytics' );
        $goal_sales  = esc_html__( 'Estimated Total Goal Value', 'qa-heatmap-analytics' );
        $goal_type   = esc_html__( 'Goal Type', 'qa-heatmap-analytics' );
        $goal_type_page  = esc_html__( 'Destination', 'qa-heatmap-analytics' );
        $goal_type_click = esc_html__( 'Click', 'qa-heatmap-analytics' );
        $goal_type_event = esc_html__( 'Event (Advanced)', 'qa-heatmap-analytics' );
        $goal_page   = esc_html__( 'Web page URL', 'qa-heatmap-analytics' );
        $click_page  = esc_html__( 'On which page?', 'qa-heatmap-analytics' );
        $eventtype   = esc_html__( 'Event Type', 'qa-heatmap-analytics' );
        $savechanges = esc_attr__( 'Save Changes', 'qa-heatmap-analytics' );
        $savesetting = esc_attr__( 'Settings saved.', 'qa-heatmap-analytics' );
        $clickselector = esc_html__( 'Click the object. (auto-fill)', 'qa-heatmap-analytics' );
        $eventselector = esc_html__( 'HTML tag (in RegEx)', 'qa-heatmap-analytics' );
        $pagematch_complete = esc_html__( 'Equals to', 'qa-heatmap-analytics' );
        $pagematch_prefix   = esc_html__( 'Begins with', 'qa-heatmap-analytics' );
        $click_sel_load = esc_html__( 'Load the Page', 'qa-heatmap-analytics' );
        $click_sel_set  = esc_html__( 'The goal has been set.', 'qa-heatmap-analytics' );
        $unset_goal = esc_html_x( 'Unset', 'unset a goal', 'qa-heatmap-analytics' );


        //each event
        $event_click   = esc_html__( 'on click', 'qa-heatmap-analytics' );

        // iframe
        $click_iframe_url = esc_url( get_home_url() );



        //1st which panel will be oepn?
        $oepndetail = array_fill(1, 2, '' );
        if ( isset( $siteinfo_ary['session_goal'] ) || isset( $siteinfo_ary['sitetype'] ) ) {
            $oepndetail[2] = 'open';
        }else{
            $oepndetail[1] = 'open';
        }

        //event measuring page
        $measuring_page_url = array();
        $is_raw_save_all   = $this->wrap_get_option( 'is_raw_save_all' );
        if ( ! $is_raw_save_all ) {
            global $qahm_recterm;
            $measuring_pages = $qahm_recterm->get_recording_page();
            for ( $iii = 0; $iii < count($measuring_pages); $iii++ ) {
                switch ( $measuring_pages[$iii]['type'] ) {
                    case 'home' :
                        $measuring_page_url[] = esc_url( get_home_url() );
                        break;
                    case 'p' :
                    case 'page_id' :
                        $measuring_page_url[] = esc_url( get_permalink( $measuring_pages[$iii]['ids'] ) );
                        break;
                    case 'cat' :
                    case 'tag' :
                    case 'tax' :
                        $measuring_page_url[] = esc_html( get_term_link( $measuring_pages[$iii]['ids'] ) );
                        break;
                }
            }
        }
        $license_plan = $this->get_plan();
	
		$acount_ary  = $qahm_google_api->get_gabackup_acount( 'gabackup_acount' );

?>

		<div id="<?php echo esc_attr( basename( __FILE__, '.php' ) ); ?>" class="qahm-admin-page">
			<div class="wrap">
                <h1>QA <?php esc_html_e( 'Settings', 'qa-heatmap-analytics' ); ?></h1>

				<?php
				// google api
				$svg_icon = '<svg xmlns="http://www.w3.org/2000/svg" width="20" height="20" viewBox="0 0 130 126"><defs><style>.cls-1{fill:#00709e;}.cls-2{fill:none;}</style></defs><path class="cls-1" d="M312.85,477.41h-1.46a2.28,2.28,0,1,1,0-4.55h.66c20.14.07,29.47-4.91,33.75-9.16a17.25,17.25,0,0,0,5.09-12.9,21.7,21.7,0,0,0-8-16.54,2.28,2.28,0,0,1-.94-1.79c-.44-20.23-9.06-33-9.15-33.14a2.3,2.3,0,0,1-.19-2.24c0-.08,3.52-7.67,4-12.64.55-6,.52-16.44-5.89-18.84l-.11-.05c-2.09-.92-8.47,5.79-11.9,9.4-5.25,5.52-8.63,8.9-11.75,8.9a2.23,2.23,0,0,1-1-.22,18.27,18.27,0,0,0-14,0,2.23,2.23,0,0,1-1,.22c-3.12,0-6.5-3.38-11.75-8.9-3.43-3.6-9.81-10.32-11.9-9.4l-.11.05c-6.41,2.4-6.44,12.83-5.89,18.84.47,5,3.95,12.56,4,12.64a2.29,2.29,0,0,1-.19,2.25c-.09.12-8.71,12.9-9.15,33.13a2.29,2.29,0,0,1-1,1.81,21,21,0,0,0-7.95,16.52,17.26,17.26,0,0,0,5.1,12.9c4.21,4.2,13.36,9.16,33,9.16h1.44a2.28,2.28,0,0,1,0,4.55H286c-17.4.07-29.87-3.41-37-10.47a21.9,21.9,0,0,1-6.44-16.13,25.62,25.62,0,0,1,9-19.5c.62-17.83,7-29.86,9.22-33.45-1.07-2.48-3.46-8.44-3.88-13-1.16-12.56,1.95-20.9,8.76-23.48,5-2.13,10.8,4,16.95,10.44,2.62,2.76,6.5,6.84,8.17,7.43a22.95,22.95,0,0,1,16.53,0c1.65-.59,5.53-4.67,8.16-7.43,6.15-6.46,12-12.57,17-10.44,6.8,2.58,9.91,10.92,8.75,23.48-.42,4.55-2.81,10.51-3.87,13,2.17,3.59,8.59,15.62,9.22,33.47a26.11,26.11,0,0,1,8.95,19.48A21.9,21.9,0,0,1,349,466.94c-7,6.95-19.16,10.47-36.15,10.47Z" transform="translate(-233 -357.5)"/><path class="cls-1" d="M270.59,398.56c-1.22-.44-3.72-7.75-4.85-12.43-1.28-5.61-1.88-12.75,2.52-14.88a3.23,3.23,0,0,1,1.4-.34c2,0,3.48,2.14,4.95,4.21.35.49.7,1,1.07,1.47a29.44,29.44,0,0,0,5.75,5.19c1.88,1.38,2.44,1.94,2,2.57-.14.21-.44.33-1.38.7a23.45,23.45,0,0,0-7.12,4c-2.86,2.45-3.51,6.37-3.83,8.25-.13.83-.18,1.06-.32,1.2a.48.48,0,0,1-.23.1Z" transform="translate(-233 -357.5)"/><path class="cls-1" d="M327.41,398.56a.4.4,0,0,1-.23-.1c-.14-.14-.19-.37-.32-1.2-.32-1.89-1-5.81-3.83-8.25a23.41,23.41,0,0,0-7.1-4c-1-.39-1.27-.5-1.4-.7-.4-.63.16-1.2,2-2.57a29.52,29.52,0,0,0,5.76-5.2c.36-.47.71-1,1.06-1.45,1.47-2.08,3-4.22,4.95-4.22a3.16,3.16,0,0,1,1.39.34c4.42,2.13,3.81,9.3,2.52,14.94-1.59,6.37-3.8,12-4.84,12.37Z" transform="translate(-233 -357.5)"/><path class="cls-1" d="M278.77,412.57s-1.53,4.08,0,5.92a7.45,7.45,0,0,0,5.24,2,8.57,8.57,0,1,1-5.25-7.93Z" transform="translate(-233 -357.5)"/><path class="cls-1" d="M325.93,412.57s-1.53,4.08,0,5.92a7.47,7.47,0,0,0,5.25,2,8.59,8.59,0,1,1-5.26-7.93Z" transform="translate(-233 -357.5)"/><path class="cls-1" d="M299,437.87c-5.84,0-10.58,3-10.58,6.63S293.15,455,299,455s10.58-6.85,10.58-10.51S304.83,437.87,299,437.87Z" transform="translate(-233 -357.5)"/><rect id="_スライス_" data-name="&lt;スライス&gt;" class="cls-2" width="130" height="126"/></svg>';
				if ( $this->wrap_get_option( 'google_is_redirect' ) ) {
					if ( $qahm_google_api->is_auth() ) {
						$auth_text = esc_html( __( 'Connected with Google API successfully.', 'qa-heatmap-analytics' ) );
					} else {
						$auth_text = esc_html( __( 'Failed to connect with Google API.', 'qa-heatmap-analytics' ) );
					}
					$this->wrap_update_option( 'google_is_redirect', false );
					echo <<< EOH
					<div class="qahm-announce-container">
						<div class="qahm-announce-icon">{$svg_icon}</div>
						<div class="qahm-announce-text">{$auth_text}</div>
					</div>
EOH;
				}
				
				$form_google_disabled = '';
				if ( $qahm_google_api->is_auth() && $this->wrap_get_option( 'gabackup_mode' ) ) {
					$form_google_disabled = ' disabled';
				}
				
				$err_ary = $qahm_google_api->test_search_console_connect();
				if ( $err_ary ) {
					$err_text  = esc_html( __( 'Failed to connect with Google API.', 'qa-heatmap-analytics' ) ) . '<br>';
					$err_text .= '<br>';
					$err_text .= 'error code: ' . $err_ary['code'] . '<br>';
					$err_text .= 'error message: ' . $err_ary['message'];
					echo '<div class="qahm-announce-container">';
					echo '<div class="qahm-announce-icon">' . $svg_icon . '</div>';
					echo '<div>' . $err_text . '</div>';
					echo '</div>';
				}
				?>
				<div class="tabs">
<!--mkdummy-->
					<input id="tab_goal" type="radio" name="tab_item" value="goal" checked>
					<label class="tab_item" for="tab_goal"><span class="qahm_margin-right4" style="pointer-events: none;"><i class="fas fa-crosshairs"></i> </span><?php esc_html_e( 'Goal', 'qa-heatmap-analytics' ); ?></label>
					<input id="tab_mail" type="radio" name="tab_item" value="mail">
					<label class="tab_item" for="tab_mail"><span class="qahm_margin-right4" style="pointer-events: none;"><i class="far fa-envelope"></i> </span><?php esc_html_e( 'Notice Mail', 'qa-heatmap-analytics' ); ?></label>
					<input id="tab_google" type="radio" name="tab_item" value="google">
					<label class="tab_item" for="tab_google"><span class="qahm_margin-right4" style="pointer-events: none;"><i class="fab fa-google"></i> </span><?php esc_html_e( 'Connect to Google', 'qa-heatmap-analytics' ); ?></label>
<!--mkdummy-->
					<div class="tab_content" id="tab_goal_content">
                        <div class="tab_content_description">
                            <p class="el_caption"><?php esc_html_e( 'You can change these goals whenever you want. This will not affect on the collected data. The value will be calculated every time.', 'qa-heatmap-analytics' ); ?></p>
                            <h2><?php esc_html_e( 'Goal setup', 'qa-heatmap-analytics' ); ?></h2>
                            <p><?php esc_html_e( 'The goal conversion rate will be on the list of each report.', 'qa-heatmap-analytics' ); ?> <span class="qahm_hatena-mark"><i class="far fa-question-circle"></i></span><a href="https://mem.quarka.org/manual/goal-setting/" target="_blank" rel="noopener"><?php esc_html_e( 'How to set goal', 'qa-heatmap-analytics' ); ?><span class="qahm_link-mark"><i class="fas fa-external-link-alt"></i></span></a></p>
                            <details id="step1" <?php echo $oepndetail[1] ?>>
                                <summary><?php esc_html_e( 'Category of your site', 'qa-heatmap-analytics' ); ?><span class="el_edit">&nbsp;<?php esc_html_e( '(Edit)', 'qa-heatmap-analytics' ); ?></span></summary>

<?php
/** ----------------------------
 * "Goal"
 */
        $formtags[0] = <<<EOL
        <form id="siteinfo_form" onsubmit="siteinfoChanges(this);return false">
        <h3>{$lang_ja['target_user_question']}</h3>
EOL;
        echo $formtags[0];

        $target_ary = ['target_individual', 'target_corporation'];
        foreach ( $target_ary as $target ) {
            $checked = '';
            if ( isset( $siteinfo_ary['target_customer'] ) ) {
                if ( $target === $siteinfo_ary['target_customer'] ) {
                    $checked = 'checked';
                }
            }
            $radio_target = <<< EOL
            <input type="radio" name="target_customer" id="{$target}" value="{$target}" $checked><label for="{$target}">{$lang_ja[$target]}</label>
EOL;
            echo $radio_target;
        }

        $formtags[1] = <<<EOL
        <h3>{$lang_ja['select_sitetype_question']}</h3>
        <table>
            <thead>
            <th>{$lang_ja['general']}</th>
            <th>{$lang_ja['media']}</th>
            <th>{$lang_ja['service']}</th>
            <th>{$lang_ja['ec_mall']}</th>
            </thead>
            <tbody>
                <tr>
EOL;
        echo $formtags[1];
        foreach ( $sitetype_ary as $lpcnt => $sitetype ) {
            $checked = '';
            if ( isset( $siteinfo_ary['sitetype'] ) ) {
                if ( $sitetype === $siteinfo_ary['sitetype'] ) {
                    $checked = 'checked';
                }
            }
            $radio_sitetype = <<< EOL
            <td><input type="radio" name="sitetype" id="{$sitetype}" value="{$sitetype}" $checked><label for="{$sitetype}">{$lang_ja[$sitetype]}</label></td>
EOL;
            $nowtd = $lpcnt + 1;
            if ( $nowtd === 14 ) {
                echo '<td>&nbsp;</td>' . PHP_EOL;
            }
            if ( 14 <= $nowtd ) {
                $nowtd++;
            }
            echo $radio_sitetype;

            if ( $nowtd % 4 === 0 ) {
                echo '</tr>' . PHP_EOL;
                if ( $nowtd !== 16 ) {
                    echo '<tr>' . PHP_EOL;
                }
            }
        }
        $formtags[2] = <<< EOL
            </tbody>
        </table>
        <h3>{$lang_ja['membership_question']}</h3>
EOL;
        echo $formtags[2];
        $membership_ary = ['membership_no', 'membership_yes'];
        foreach ( $membership_ary as $membership ) {
            $checked = '';
            if ( isset( $siteinfo_ary['membership'] ) ) {
                if ( $membership === $siteinfo_ary['membership'] ) {
                    $checked = 'checked';
                }
            }
            $radio_membership = <<< EOL
            <input type="radio" name="membership" id="{$membership}" value="{$membership}" $checked><label for="{$membership}">{$lang_ja[$membership]}</label>
EOL;
            echo $radio_membership;
        }

        $formtags[3] = <<< EOL
        <h3>{$lang_ja['payment_question']}</h3>
EOL;
        echo $formtags[3];
        $payment_ary = ['payment_no', 'payment_yes', 'payment_cart'];
        foreach ( $payment_ary as $payment ) {
            $checked = '';
            if ( isset( $siteinfo_ary['payment'] ) ) {
                if ( $payment === $siteinfo_ary['payment'] ) {
                    $checked = 'checked';
                }
            }
            $radio_payment = <<< EOL
            <input type="radio" name="payment" id="{$payment}" value="{$payment}" $checked><label for="{$payment}">{$lang_ja[$payment]}</label>
EOL;
            echo $radio_payment;
        }

        $month_later  = isset( $siteinfo_ary['month_later'] )? $siteinfo_ary['month_later'] : '';
        $session_goal = isset( $siteinfo_ary['session_goal'] )? $siteinfo_ary['session_goal'] : '';
        $formtags[4] = <<< EOL
        <h3>{$lang_ja['goal_monthly_access_question']}</h3>
        <input type="number" name="month_later" id="month_later" value="{$month_later}"><label for="month_later">{$lang_ja['month_later']}</label>&nbsp;
        <input type="number" name="session_goal" id="session_goal" value="{$session_goal}"><label for="session_goal">{$lang_ja['session_goal']}</label>
        <p><input type="submit" value="{$lang_ja['next']}" onclick="detailopen(2)"></p>
        </form>
EOL;
        echo $formtags[4];

        echo '</details>';
        echo '<details id="step2" ' . $oepndetail[2] . '>';
        echo '<summary>' . esc_html__( 'Set goals', 'qa-heatmap-analytics' ) . '<span class="el_edit">&nbsp;' . esc_html__( '(Edit)', 'qa-heatmap-analytics' ) . '</span></summary>';

        $gtype_iframe_display  = array_fill(1, $GOALMAX,  'style="display: none"' );
        for ( $iii = 1; $iii <= $GOALMAX; $iii++ ) {
            $gtitle = isset( $goals_ary[$iii]['gtitle'] ) ? urldecode( $goals_ary[$iii]['gtitle'] ) : '';
            $gnum_scale = isset( $goals_ary[$iii]['gnum_scale'] ) ? urldecode( $goals_ary[$iii]['gnum_scale'] ) : 0;
            $gnum_value = isset( $goals_ary[$iii]['gnum_value'] ) ? urldecode( $goals_ary[$iii]['gnum_value'] ) : 0;
            $gtype = isset( $goals_ary[$iii]['gtype'] ) ? urldecode( $goals_ary[$iii]['gtype'] ) : 'gtype_page';
            $g_goalpage = isset( $goals_ary[$iii]['g_goalpage'] ) ? urldecode( $goals_ary[$iii]['g_goalpage'] ) : '';
            $g_pagematch = isset( $goals_ary[$iii]['g_pagematch'] ) ? urldecode( $goals_ary[$iii]['g_pagematch'] ) : '';
            $g_clickpage = isset( $goals_ary[$iii]['g_clickpage'] ) ? urldecode( $goals_ary[$iii]['g_clickpage'] ) : '';
            $g_eventtype = isset( $goals_ary[$iii]['g_eventtype'] ) ? urldecode( $goals_ary[$iii]['g_eventtype'] ) : '';
            $g_clickselector = isset( $goals_ary[$iii]['g_clickselector'] ) ? urldecode( $goals_ary[$iii]['g_clickselector'] ) : '';
            $g_eventselector = isset( $goals_ary[$iii]['g_eventselector'] ) ? urldecode( $goals_ary[$iii]['g_eventselector'] ) : '';


            $gtype_checked  = array_fill(0, 3, '');
            $gtype_required = array_fill(0, 3, '');
            $pagematch_checked  = array_fill(0, 2, '');
            $gtype_display  = array_fill(0, 3, 'style="display: none"');

            //set default
            if ( ! $g_clickpage ) {
                if ( $is_raw_save_all ) {
                    $g_clickpage = $click_iframe_url;
                } else {
                    $g_clickpage = $measuring_page_url[0];
                }
            }
            switch ($gtype) {
                case 'gtype_click':
                    $gtype_checked[1]  ='checked';
                    $gtype_required[1] = 'required';
                    $gtype_iframe_display[$iii]  = '';
                    $gtype_display[1] = '';
                    break;

                case 'gtype_event':
                    $gtype_checked[2]  ='checked';
                    $gtype_required[2] = 'required';
                    $gtype_display[2] = '';
                    break;

                default:
                case 'gtype_page':
                    $gtype_checked[0]  ='checked';
                    $gtype_required[0] = 'required';
                    $gtype_display[0] = '';
                    break;
            }
            switch ($g_pagematch) {
                case 'pagematch_prefix':
                    $pagematch_checked[ 1 ] = 'checked';
                    break;

                default:
                case 'pagematch_complete':
                    $pagematch_checked[ 0 ] = 'checked';
                    break;
            }

            //when goal type = "click", select box for limited event-measuring page.
            if ( ! $is_raw_save_all ) {
                $clicktype_page_input = '<select name="g'.$iii.'_clickpage" id="g'.$iii.'_clickpage" '.$gtype_required[1].'>';
                $clicktype_page_input .= '<option value="'.$g_clickpage.'">' . $g_clickpage . '</option>';
                for ( $jjj = 0; $jjj < count($measuring_page_url); $jjj++ ) {
                    if ( $measuring_page_url[$jjj] == $g_clickpage ) {
                        continue;
                    }
                    $clicktype_page_input .= '<option value="'.$measuring_page_url[$jjj].'">' . $measuring_page_url[$jjj] . '</option>';
                }
                $clicktype_page_input .= '</select>';
                $attention_only_measuring_page = '<p><span class="qahm_bikkuri-mark"><i class="fas fa-exclamation-circle"></i></span>';
                $attention_only_measuring_page .= esc_html__( 'Settable only the event measuring page.', 'qa-heatmap-analytics' );
                if ( ! $license_plan ) {
                    $attention_only_measuring_page .= ' (<a href="https://quarka.org/dataplan-upgrade/" target="_blank" rel="noopener">';
                    $attention_only_measuring_page .= esc_html__( 'Upgrade and measure event on all the pages.', 'qa-heatmap-analytics' );
                    $attention_only_measuring_page .= '<span class="qahm_link-mark"><i class="fas fa-external-link-alt"></i></a>)';
                }
                $attention_only_measuring_page .= '</p>';
            } else {
                $clicktype_page_input = '<input type="text" name="g'.$iii.'_clickpage" id="g'. $iii.'_clickpage" value="'.$g_clickpage.'" '. $gtype_required[1].' placeholder="'.$click_iframe_url.'" size="40">';
                $attention_only_measuring_page = '';
            }



            $goalbox = <<<EOL
                <div class="bl_goalbox" id="g{$iii}_goalbox">
                    <h3>{$goal}{$iii}</h3>
                    <form id="g{$iii}_form" onsubmit="saveChanges(this);return false">
                    <table>
                        <colgroup>
                            <col style="width: 15%">
                            <col style="width: 65%">
                            <col style="width: 20%">
                        <tbody>
                        <tr>
                            <td>{$goal_title}<span class="el_attention">{$required}</span></td>
                            <td><input type="text" name="g{$iii}_title" id="g{$iii}_title" required value="{$gtitle}" size="30"></td>
                            <td>&nbsp;</td>
                        </tr>
                        <tr>
                            <td>{$goal_number}</td>
                            <td><input type="number" name="g{$iii}_num" id="g{$iii}_num" value="{$gnum_scale}" onchange="calcSales(this)">{$num_scale}</td>
                            <td>&nbsp&nbsp;</td>
                        </tr>
                        <tr>
                            <td>{$goal_value}</td>
                            <td><input type="number" name="g{$iii}_val" id="g{$iii}_val" value="{$gnum_value}" onchange="calcSales(this)">{$val_scale}&nbsp;<p class="right">{$goal_sales} = <span id="g{$iii}_calcsales">0</span> {$val_scale}</p></td>
                            <td>&nbsp;</td>
                        </tr>
                        <tr>
                            <td>{$goal_type}<span class="el_attention">{$required}</span>&nbsp;<span class="el_loading">Loading<span></span></span></td>
                            <td class="td_gtype_save" style="opacity: 0">
                                <input type="radio" name="g{$iii}_type" id="g{$iii}_type_page" value="gtype_page" {$gtype_checked[0]}><label for="g{$iii}_type_page">{$goal_type_page}</label>
                                <input type="radio" name="g{$iii}_type" id="g{$iii}_type_click" value="gtype_click" {$gtype_checked[1]}><label for="g{$iii}_type_click">{$goal_type_click}</label>&nbsp;
                                <span  style="opacity: 0"><input type="radio" name="g{$iii}_type" id="g{$iii}_type_event" value="gtype_event" {$gtype_checked[2]}><label for="g{$iii}_type_event">{$goal_type_event}</label></span>&nbsp;
                                <br>
                                <div id="g{$iii}_page_goal" ${gtype_display[0]} class="bl_eachGtypeBox">
                                    <label>{$goal_page}</label><input type="text" name="g{$iii}_goalpage" id="g{$iii}_goalpage" value="{$g_goalpage}" {$gtype_required[0]} size="40">
                                    <input type="radio" name="g{$iii}_pagematch" id="g{$iii}_pagematch_prefix" value="pagematch_prefix" {$pagematch_checked[1]}><label for="g{$iii}_pagematch_prefix">{$pagematch_prefix}</label>
                                    <input type="radio" name="g{$iii}_pagematch" id="g{$iii}_pagematch_complete" value="pagematch_complete" {$pagematch_checked[0]}><label for="g{$iii}_pagematch_complete">{$pagematch_complete}</label>&nbsp;
                                </div>
                                <div id="g{$iii}_click_goal" ${gtype_display[1]} class="bl_eachGtypeBox">

                                    <label>{$click_page}</label>{$clicktype_page_input}
                                    <button id="g{$iii}_click_pageload" class="button button-secondary" type="button">{$click_sel_load}</button><br>
                                    {$attention_only_measuring_page}
                                    <label>{$clickselector}</label><input type="text" name="g{$iii}_clickselector" id="g{$iii}_clickselector" disabled value="{$g_clickselector}" {$gtype_required[1]} size="35">
                                    <div id="g{$iii}_event-iframe-tooltip-right" class="event-iframe-tooltip-right">{$click_sel_set}</div>
                                </div>
                                <div id="g{$iii}_event_goal" ${gtype_display[2]} class="bl_eachGtypeBox">
                                    <label>{$eventtype}</label><select name="g{$iii}_eventtype" id="g{$iii}_eventtype"><option value="{$event_click}">{$event_click}</option></select> <br>
                                    <label>{$eventselector}</label><input type="text" name="g{$iii}_eventselector" id="g{$iii}_eventselector" value="{$g_eventselector}" {$gtype_required[2]}>
                                </div>
                            </td>
                            <td class="td_gtype_save" style="opacity: 0"><input type="submit" name="submit" id="g{$iii}_submit" value="{$savechanges}" class="button button-primary"><p class="el_right"><a href="#g{$iii}_goalbox" onclick="deleteGoalX({$iii})">{$unset_goal}</a></p></td>
                        </tr>
                        </tbody>
                    </table>
                    <div id="g{$iii}_event-iframe-containar" class=".event-iframe-containar" {$gtype_iframe_display[$iii]}>
                        <iframe id="g{$iii}_event-iframe" class="event-iframe" src="{$g_clickpage}" frameborder="0" width="1200" height="400" scrolling="yes"></iframe>
                    </div>
                    </form>
                </div>
EOL;
            echo $goalbox;
        }
        //end for
        $goal_tab_final = <<< EOL
                                </details>
                            </div>
                        </div><!-- endof #tab_goal_content -->                        
EOL;
        echo $goal_tab_final;

?>
<?php
/** --------------------------------
 * "Mail"
 */
        $register    = esc_attr__( 'Register', 'qa-heatmap-analytics' );
        //$savechanges = esc_attr__( 'Save Changes', 'qa-heatmap-analytics' );
        $registerbtn   = $register;
        $already_registered = false;
        $email = get_option('admin_email');
        $popmail_cycle = '';
        $popmail_day = '';
        global $qahm_data_api;
        $email_notice_ary = $qahm_data_api->get_email_notice_ary();
        if ( array_key_exists('regist', $email_notice_ary ) ) {
            if ( $email_notice_ary['regist'] ) {
                //$registerbtn = $savechanges;
                $already_registered = true;
            }
            if ( filter_var( $email_notice_ary['email'], FILTER_VALIDATE_EMAIL ) ) {
                $email = $email_notice_ary['email'];
            }
        }
?>
                    <div class="tab_content" id="tab_mail_content">
                        <div class="bl_whitediv">
                            <div style="width: 800px">

                                <div class="mail_config_section">
                                    <h2><span class="mail_config_fa"><i class="far fa-paper-plane"></i></span><?php esc_html_e( 'Automatic Notice Mail', 'qa-heatmap-analytics' ); ?></h2>
                                    <div class="mail_config_inputpart">
                                        <p><?php esc_html_e( 'Notice email sent automatically to the WordPress administrator.', 'qa-heatmap-analytics' ); ?><br><?php esc_html_e( 'It notifies you when data maintenance is finished after updating the plugin.', 'qa-heatmap-analytics' ); ?></p>
                                        
                                        <form onsubmit="configMailSubmit(this);return false">
                                            <h4 class="mail_config_toph4"><?php esc_html_e( 'Send to:', 'qa-heatmap-analytics' ); ?></h4>
                                            <input type="email" name="email" placeholder="you@example.com" value="<?php echo $email; ?>" required>
                                            <input type="hidden" name="is_config_page" value="true">
                                            <?php if ( $already_registered ) { ?>                                          
                                                <input type="hidden" name="is_regist_post" value="false">                                                
                                                <p class="mail_config_debtn"><input type="submit" name="submit" id="" value="<?php esc_attr_e( 'Deregister', 'qa-heatmap-analytics' ); ?>" class="button button-primary"></p>
                                            <?php } else { ?>
                                                <input type="hidden" name="is_regist_post" value="true">
                                                <p class="mail_config_sbtn"><input type="submit" name="submit" id="" value="<?php echo $registerbtn; ?>" class="button button-primary"></p>
                                            <?php } ?>
                                        </form>
                                        

                                    </div>
                                </div>

                                <div class="mail_config_section">
                                    <h2><span class="mail_config_fa"><i class="far fa-newspaper"></i></span> <?php echo esc_html__( 'QA Newsletter' ); ?> </h2>
                                    <div class="mail_config_inputpart">
                                        <p><?php echo esc_html__( 'Receive our Free Newsletters (in Japanese) with website improvement case studies and QA Analytics information.', 'qa-heatmap-analytics' ); ?></p>
                                        <p><a href="https://mem.quarka.org:7383/_mlists/?p=subscribe&id=1&email=<?php echo $email; ?>" target="_blank" rel="noopener" id="linkto_nl_btn"> <?php echo esc_html__( 'Newsletter Admin' ); ?> </a></p>
                                    </div>
                                </div>

                                <p><span class="qahm_hatena-mark"><i class="far fa-question-circle"></i></span><a href="https://mem.quarka.org/manual/about-auto-alert-mail/" target="_blank" rel="noopener"><?php echo esc_html__( 'Automatic Notice Mail', 'qa-heatmap-analytics' ); ?><span class="qahm_link-mark"><i class="fas fa-external-link-alt"></i></span></a></p>

                            </div>
                        </div>
                    </div><!-- endof #tab_mail_content -->
<?php
/** --------------------------------
 * "Google API"
 */
?>
                        <div class="tab_content" id="tab_google_content">
                            <h1><?php echo esc_html__( 'Connect with Google API', 'qa-heatmap-analytics' ); ?></h1>
                            <p>
								<?php echo esc_html( __( 'API integration with Google allows you to retrieve data from Google Search Console and Google Analytics.', 'qa-heatmap-analytics' ) ); ?>
								<span class="qahm_hatena-mark"><i class="far fa-question-circle"></i></span>
								<a href="https://mem.quarka.org/manual/connect-to-gsc/" target="_blank" rel="noopener"><?php echo esc_html( __( 'How to connect with API', 'qa-heatmap-analytics' ) ); ?><span class="qahm_link-mark"><i class="fas fa-external-link-alt"></i></span></a>
							</p>
                            <form method="post" action="">
                                <?php wp_nonce_field( self::NONCE_ACTION, self::NONCE_NAME, false ); ?>

                                <table class="form-table">
                                    <tbody>
                                        <tr>
                                            <th scope="row">
                                                <label for="client_id">
													<?php echo esc_html( __( 'Client ID', 'qa-heatmap-analytics' ) ); ?>
                                                </label>
                                            </th>
                                            <td>
                                                <input name="client_id" type="text" id="client_id" value="<?php echo esc_attr( $qahm_google_api->get_client_id() ); ?>" class="regular-text"<?php echo $form_google_disabled; ?>>
                                            </td>
                                        </tr>

                                        <tr>
                                            <th scope="row">
                                                <label for="client_secret">
													<?php echo esc_html( __( 'Client Secret', 'qa-heatmap-analytics' ) ); ?>
                                                </label>
                                            </th>
                                            <td>
                                                <input name="client_secret" type="text" id="client_secret" value="<?php echo esc_attr( $qahm_google_api->get_client_secret() ); ?>" class="regular-text"<?php echo $form_google_disabled; ?>>
												<?php
												if ( $form_google_disabled !== '' ) {
													echo '<span id="client_info_disabled_text" style="font size: 0.9em; color: #2271b1; cursor: pointer; text-decoration: underline;">&nbsp;' . esc_html( $this->japan( 'disabledを解除する' , 'qa-heatmap-analytics' ) ) . '</span>';
												}
												?>
												</td>
                                        </tr>

                                        <tr>
                                            <th scope="row">
                                                <label for="redirect_uri">
													<?php echo esc_html( __( 'Redirect URI', 'qa-heatmap-analytics' ) ); ?>
                                                </label>
                                            </th>
                                            <td>
                                                <p><?php echo esc_attr( admin_url( 'admin.php?page=qahm-config' ) ); ?></p>
                                            </td>
                                        </tr>

										<?php
											if ( $acount_ary ) {
												for ( $acount_idx = 0, $acount_max = count( $acount_ary ); $acount_idx < $acount_max; $acount_idx++ ) {
													?>
													<tr>
														<th scope="row">
															<label for="redirect_uri">
																<?php echo esc_html( __( 'E-mail', 'qa-heatmap-analytics' ) ) . ' ' . ( $acount_idx + 1 ); ?>
															</label>
														</th>
														<td>
															<p><?php echo $acount_ary[$acount_idx]['mail']; ?></p>
														</td>
													</tr>
													<?php
												}
											}
										?>

                                        <tr>
                                            <td colspan="2">
                                                <p class="submit" style="text-align: center">
                                                    <input type="submit" name="submit" id="submit" class="button button-primary" value="<?php esc_attr_e( 'Authenticate', 'qa-heatmap-analytics' ); ?>">
                                                </p>
                                            </td>
                                        </tr>
                                    </tbody>
                                </table>
                            </form>
                           
                    	</div><!-- endof #tab_google_content -->
            </div>
        </div>
    </div>


<?php
	}

	/**
	 * 設定画面の項目をデータベースに保存する
	 */
	public function save_config() {
		$post = array();
		// nonceで設定したcredentialのチェック
		// 設定画面
		if ( isset( $_POST[ self::NONCE_NAME ] ) ) {
			if ( check_admin_referer( self::NONCE_ACTION, self::NONCE_NAME ) ) {
				$license_plan = $this->get_plan();

				if ( $license_plan ) {
					$is_raw_save_all = $this->wrap_filter_input( INPUT_POST, 'is_raw_save_all' );
					$access_role     = $this->wrap_filter_input( INPUT_POST, 'access_role' );

					$this->wrap_update_option( 'is_raw_save_all', $is_raw_save_all );

					if ( $access_role !== null ) {
						$this->wrap_update_option( 'access_role', $access_role );
					}
				}
			}
		}
	}
} // end of class
