<?php
/**
 *
 *
 * @package qa_heatmap
 */

$qahm_admin_page_heatmap = new QAHM_Admin_Page_Heatmap();

class QAHM_Admin_Page_Heatmap extends QAHM_Admin_Page_Base {

	// スラッグ
	const SLUG = QAHM_NAME . '-heatmap';

	// nonce
	const NONCE_ACTION  = self::SLUG . '-nonce-action';
	const NONCE_NAME    = self::SLUG . '-nonce-name';
	const NONCE_REFRESH = 'refresh';

	private static $error_msg = array();
	private $license_plan;

	/**
	 * コンストラクタ
	 */
	public function __construct() {
		parent::__construct();

		$this->regist_ajax_func( 'ajax_refresh_version' );
		$this->regist_ajax_func( 'ajax_create_heatmap_list' );
		$this->regist_ajax_func( 'ajax_set_achievements' );
		$this->regist_ajax_func( 'ajax_change_is_raw_save_all' );
		$this->regist_ajax_func( 'ajax_get_data_num' );
		$this->regist_ajax_func( 'ajax_create_unmeasurable_table' );

		add_action( 'init', array( $this, 'init_wp_filesystem' ) );
		add_action( 'admin_init', array( $this, 'save_config' ) );
	}

	/**
	 * 初期化
	 */
	public function enqueue_scripts( $hook_suffix ) {
		if( $this->hook_suffix !== $hook_suffix ||
			! $this->is_enqueue_jquery()
		) {
			return;
		}

		$css_dir_url = $this->get_css_dir_url();
		$js_dir_url  = $this->get_js_dir_url();

		// enqueue_style
		$this->common_enqueue_style();
		wp_enqueue_style( QAHM_NAME . '-admin-page-heatmap', $css_dir_url . 'admin-page-heatmap.css', array( QAHM_NAME . '-reset' ), QAHM_PLUGIN_VERSION );	
		wp_enqueue_style( QAHM_NAME . '-table', $css_dir_url . 'table.css', array( QAHM_NAME . '-reset' ), QAHM_PLUGIN_VERSION );	
	
		// enqueue script
		$this->common_enqueue_script();
		wp_enqueue_script( QAHM_NAME . '-table', $js_dir_url . 'table.js', null, QAHM_PLUGIN_VERSION );
		wp_enqueue_script( QAHM_NAME . '-progress-bar', $js_dir_url . '/progress-bar-exec.js', null, QAHM_PLUGIN_VERSION );
		wp_enqueue_script( QAHM_NAME . '-cap-create', $js_dir_url . 'cap-create.js', array( QAHM_NAME . '-admin-common' ), QAHM_PLUGIN_VERSION );
		wp_enqueue_script( QAHM_NAME . '-admin-page-heatmap', $js_dir_url . 'admin-page-heatmap.js', array( QAHM_NAME . '-admin-common' ), QAHM_PLUGIN_VERSION );
	
		// inline script
		$scripts = $this->get_common_inline_script();
		$scripts['heatmap_measure_max']      = $this->wrap_get_option( 'heatmap_measure_max' );
		$scripts['is_raw_save_all']          = $this->wrap_get_option( 'is_raw_save_all' );
		$scripts['achievements']             = $this->wrap_get_option( 'achievements' );
		$scripts['nonce_refresh']            = wp_create_nonce( self::NONCE_REFRESH );
		$scripts['is_first_heatmap_setting'] = $this->wrap_get_option( 'is_first_heatmap_setting' );
		wp_add_inline_script( QAHM_NAME . '-common', 'var ' . QAHM_NAME . ' = ' . QAHM_NAME . ' || {}; let ' . QAHM_NAME . 'Obj = ' . wp_json_encode( $scripts ) . '; ' . QAHM_NAME . ' = Object.assign( ' . QAHM_NAME . ', ' . QAHM_NAME . 'Obj );', 'before' );

		// localize
		$localize = $this->get_common_localize_script();
		$localize['home']                             = esc_html__( 'Home', 'qa-heatmap-analytics' );
		$localize['post']                             = esc_html__( 'Post', 'qa-heatmap-analytics' );
		$localize['page']                             = esc_html__( 'Page', 'qa-heatmap-analytics' );
		$localize['cat']                              = esc_html__( 'Category', 'qa-heatmap-analytics' );
		$localize['tag']                              = esc_html__( 'Tag', 'qa-heatmap-analytics' );
		$localize['tax']                              = esc_html__( 'Custom taxonomy', 'qa-heatmap-analytics' );
		$localize['ver']                              = esc_html__( 'ver.', 'qa-heatmap-analytics' );
		$localize['limit_measure_title']              = esc_html__( 'About measurement', 'qa-heatmap-analytics' );
		$localize['limit_measure_text']               = esc_html__( 'Measurable for heatmap is limited to %d page(s) with your QA.', 'qa-heatmap-analytics' ) . '<br>';
		$localize['limit_free_text']                  = sprintf( esc_html__( '%1$s Upgrade %2$s to collect event data on more pages.', 'qa-heatmap-analytics' ), '<a href="https://quarka.org/dataplan-upgrade/" target="_blank" rel="noopener">', '</a>' );
		//$localize['limit_friend_text']                = sprintf( esc_html__( '%1$s Get upgrade options %2$s.', 'qa-heatmap-analytics' ), '<a href="https://quarka.org/dataplan-upgrade/" target="_blank" rel="noopener">', '</a>' );
		$localize['switch']                           = esc_html__( 'switched', 'qa-heatmap-analytics' );
		$localize['version_switch']                   = esc_html__( 'Switch the Page Version', 'qa-heatmap-analytics' );
		$localize['version_switch_error_text']        = sprintf( esc_html__( 'Failed to switch the version. %s Please try again aftertime.', 'qa-heatmap-analytics' ), '<br>' );
		$localize['version_switch_start_text_1']      = esc_html__( 'Save the current page version and start a new measurement.', 'qa-heatmap-analytics' );
		$localize['version_switch_start_text_2']      = esc_html__( 'Use this after updated the page (post) or edited the html.', 'qa-heatmap-analytics' );
		$localize['version_switch_inte_title']        = esc_html__( 'Data is in the process of integration', 'qa-heatmap-analytics' );
		$localize['version_switch_inte_text']         = esc_html__( 'Sorry, switching the pageversion is prevented.<br>This page is being combined to the data by QA version 3.0. Please wait until the data gets ready.', 'qa-heatmap-analytics' );
		$localize['achievements_data_num_1_title']    = esc_html__( 'Now on taking the first step!', 'qa-heatmap-analytics' );
		$localize['achievements_data_num_1_text']     = sprintf( esc_html__( 'Congratulations! %s Data has been collected for the first time.', 'qa-heatmap-analytics' ), '<br>' );
		$localize['achievements_data_num_1000_title'] = esc_html__( 'Improved analytical skills!', 'qa-heatmap-analytics' );
		$localize['achievements_data_num_1000_text']  = esc_html__( 'In total, %d data have been collected in QA!', 'qa-heatmap-analytics' );
		$localize['measure_comment_text']             = esc_html__( 'You have reached the limit of measuable pages.', 'qa-heatmap-analytics' );
		$localize['achievements_data_num_1000_text']  = esc_html__( 'In total, %d data have been collected in QA!', 'qa-heatmap-analytics' );
		$localize['page_title']                       = esc_html__( 'Page Title', 'qa-heatmap-analytics' );
		$localize['post_type']                        = esc_html__( 'Post Type', 'qa-heatmap-analytics' );
		wp_localize_script( QAHM_NAME . '-common', QAHM_NAME . 'l10n', $localize );

		$this->license_plan = (int) $this->wrap_get_option( 'license_plan' );
	}


	/**
	 * ページの表示
	 */
	public function create_html() {
		if( ! $this->is_enqueue_jquery() ) {
			$this->view_not_enqueue_jquery_html();
			return;
		}

		if( ! $this->is_email_regist() ) {
			$this->view_noticemail_html();
			return;
		}

		if( $this->is_maintenance() ) {
			$this->view_maintenance_html();
			return;
		}

		// データを取得
		$is_raw_save_all   = $this->wrap_get_option( 'is_raw_save_all' );
		$heatmap_sort_rec  = $this->wrap_get_option( 'heatmap_sort_rec' );
		$heatmap_sort_view = $this->wrap_get_option( 'heatmap_sort_view' );
		?>

		<div id="<?php echo esc_attr( basename( __FILE__, '.php' ) ); ?>" class="qahm-admin-page">
			<div class="wrap">
				<h1>QA <?php esc_html_e( 'Heatmap Manager', 'qa-heatmap-analytics' ); ?></h1>
				<?php 
				$license_plan = $this->get_plan();
				if ( ! $license_plan ) {
                    $msg_yuryouikaga = '<div class="qahm_using_free_announce"><span class="qahm_margin-right4"><i class="fas fa-exclamation-triangle"></i></span><span class="qahm_fontsize12em">';
                    $msg_yuryouikaga .=	sprintf( esc_html__( 'You are using the FREE version of QA. Get full-fledged and enrich analysis with %1$s QA Upgrade Options %2$s.', 'qa-heatmap-analytics' ), '<a href="https://quarka.org/dataplan-upgrade/" target="_blank" rel="noopener">', '</a>' ); //'現在『無料版』をご利用中です。QA有料機能を使えば、より本格的で充実した解析ができます。'
                    $msg_yuryouikaga .= '</span></div>';
                    echo $msg_yuryouikaga;
				}
				?>

				<?php
					// メッセージ表示
					if ( ! empty( self::$error_msg ) ) {
						foreach ( self::$error_msg as $msg ) {
							echo '<div class="error notice is-dismissible"><p><strong>' . esc_html__( 'Error', 'qa-heatmap-analytics' ) . ' : ' . esc_html( $msg ) . '</strong></p></div>';
						}
					} elseif ( isset( $_POST[ self::NONCE_NAME ] ) ) {
						echo '<div class="updated notice is-dismissible"><p><strong>' . esc_html__( 'Saved settings.', 'qa-heatmap-analytics' ) . '</strong></p></div>';
					}
				?>

				<h2><?php esc_html_e( 'Data and Measurement Settings of Your Heatmap', 'qa-heatmap-analytics' ); ?></h2>
				<p>
					<?php
						esc_html_e( 'Select a checkbox on the column of the page which you want to measure event and save the changes.', 'qa-heatmap-analytics' );
						echo '<br><span class="qahm_bikkuri-mark"><i class="fas fa-exclamation-circle"></i></span>';
						esc_html_e( 'You can view the heatmap data from the day after the measurement is enabled.', 'qa-heatmap-analytics' );
					?>
				</p>

				<form action="" method="post" id="qahm-data-sort-form" onsubmit="qahm.createList(); return false;">
					<div class="tablenav top">
						<div class="alignleft">
							<input type="text" id="qahm-data-sort-title" placeholder="<?php esc_attr_e( 'Title to search', 'qa-heatmap-analytics' ); ?>" class="regular-text">
						</div>

						<select id="qahm-data-sort-rec">
							<option value="all" 
								<?php
								if ( $heatmap_sort_rec === 'all' ) {
									echo ' selected';
								}
								?>
							>
								<?php esc_html_e( 'All pages', 'qa-heatmap-analytics' ); ?>
							</option>

							<option value="true" 
								<?php
								if ( $heatmap_sort_rec === 'true' ) {
									echo ' selected';
								}
								?>
							>
								<?php esc_html_e( 'Pages being measured', 'qa-heatmap-analytics' ); ?>
							</option>
							<option value="false" 
								<?php
								if ( $heatmap_sort_rec === 'false' ) {
									echo ' selected';
								}
								?>
							>
								<?php esc_html_e( 'Pages NOT being measured', 'qa-heatmap-analytics' ); ?>
							</option>
						</select>

						<select id="qahm-data-sort-view">
							<option value="10"
								<?php
								if ( $heatmap_sort_view === '10' ) {
									echo esc_attr( ' selected' );
								}
								?>
							>
								<?php printf( esc_html__( 'Show %d', 'qa-heatmap-analytics' ), 10 ); ?>
							</option>

							<option value="25"
								<?php
								if ( $heatmap_sort_view === '25' ) {
									echo esc_attr( ' selected' );
								}
								?>
							>
								<?php printf( esc_html__( 'Show %d', 'qa-heatmap-analytics' ), 25 ); ?>
							</option>

							<option value="50"
								<?php
								if ( $heatmap_sort_view === '50' ) {
									echo esc_attr( ' selected' );
								}
								?>
							>
								<?php printf( esc_html__( 'Show %d', 'qa-heatmap-analytics' ), 50 ); ?>
							</option>

							<option value="100"
								<?php
								if ( $heatmap_sort_view === '100' ) {
									echo esc_attr( ' selected' );
								}
								?>
							>
								<?php printf( esc_html__( 'Show %d', 'qa-heatmap-analytics' ), 100 ); ?>
							</option>
							
							<option value="all"
								<?php
								if ( $heatmap_sort_view === 'all' ) {
									echo esc_attr( ' selected' );
								}
								?>
							>
								<?php esc_html_e( 'Show ALL', 'qa-heatmap-analytics' ); ?>
							</option>
						</select>

						<input type="submit" id="qahm-data-sort-button" class="button" value="<?php esc_attr_e( 'Get Search', 'qa-heatmap-analytics' ); ?>">
						<div class="alignright">
							<p>
								<label for="qahm-all-rec-flag">
									<input type="checkbox" id="qahm-all-rec-flag"<?php if( $is_raw_save_all ){ echo ' checked'; } ?>>
									<?php esc_html_e( 'Measure all of the pages automatically, including newly published pages/posts.', 'qa-heatmap-analytics' ); ?>
								<label>
							</p>
						</div>
						<br class="clear">
					</div>
				</form>

				<form action="" method="post" id="dataForm" name="dataForm">
					<?php // ②：nonceの設定 ?>
					<?php wp_nonce_field( self::NONCE_ACTION, self::NONCE_NAME ); ?>
					
					<table id="qahm-data-list" class="widefat striped">
						<thead>
							<tr>
								<th scope="col">
									<?php esc_html_e( 'Page Title', 'qa-heatmap-analytics' ); ?>
								</th>
								<th scope="col">
									<?php esc_html_e( 'Heatmap (Amount of Data)', 'qa-heatmap-analytics' ); ?>
									<span class="qahm-tooltip" data-qahm-tooltip="<?php echo(
										esc_attr__( 'Select the icon of each device to see the heatmap.', 'qa-heatmap-analytics' )
										); ?>">
										<i class="far fa-question-circle"></i>
									</span>
								</th>
								<th scope="col">
									<?php
										esc_html_e( 'Page Version', 'qa-heatmap-analytics' );
										echo ' : ';
										esc_html_e( 'Span', 'qa-heatmap-analytics' );
									?>
								</th>
								<th scope="col" id="test">
									<?php esc_html_e( 'Switch Version', 'qa-heatmap-analytics' ); ?>
									<span class="qahm-tooltip" data-qahm-tooltip="<?php echo(
										esc_attr__( 'Save the current page version and start a new measurement. ', 'qa-heatmap-analytics' ) .
										esc_attr__( 'Use this after updated the page (post) or edited the html.', 'qa-heatmap-analytics' )
										); ?>">
										<i class="far fa-question-circle"></i>
									</span>
								</th>
								<th scope="col">
									<?php esc_html_e( 'Type', 'qa-heatmap-analytics' ); ?>
								</th>
								<th scope="col"<?php
									if( $is_raw_save_all ){
										echo ' style="' . esc_attr( 'background-color: #dc3545; color: #fff;' ) . '"';
									}
									?>>
									<?php 
									if ( $is_raw_save_all ) {
										esc_html_e( 'Measuring All Pages.', 'qa-heatmap-analytics' );
									} else {
										esc_html_e( 'Measuring', 'qa-heatmap-analytics' );
									}
									?>
								</th>
							</tr>
						</thead>
						<tbody>
						</tbody>
					</table>

					<p class="submit">
						<input type="submit" name="submit" id="submit" value="<?php esc_attr_e( 'Save Changes', 'qa-heatmap-analytics' ); ?>" onclick="qahm.postParam();" class="button button-primary">
					</p>
				</form>
			
				<div id="unmeasurable-container" style="display:none;">
					<?php if ( $this->get_plan() ) { ?>
						<h3><?php esc_html_e( 'Use the QA add-on for special post types.', 'qa-heatmap-analytics' ); ?></h3>
						<p><?php esc_html_e( 'Post types established by a theme template, that are not standard WordPress posts, can be measured by adding a special QA add-on, which you can get free.', 'qa-heatmap-analytics' ); ?><br>
						<?php esc_html_e( 'This will be on a case-by-case basis, as it needs to match the theme you are using.', 'qa-heatmap-analytics' ); ?><br>
						<?php printf( esc_html__( 'Please %1$s contact us %2$s from our support site. We will make an add-on and send it to you.', 'qa-heatmap-analytics' ), '<a href="https://mem.quarka.org/memberhome/" target="_blank" rel="noopener noreferrer">', '</a>' ); ?>
					<?php } else { ?>
						<h3><?php esc_html_e( 'Can be measured with a subscription plan.', 'qa-heatmap-analytics' ); ?></h3>
						<p><?php esc_html_e( 'Post types established by a theme template, that are not standard WordPress posts, can be measured by adding a special QA add-on that is available for subscription plans.', 'qa-heatmap-analytics' ); ?><br>
						<?php printf( esc_html__( 'To see %1$s Subscription Plan %2$s .', 'qa-heatmap-analytics' ), '<a href="https://quarka.org/plan/" target="_blank" rel="noopener noreferrer">', '</a>' ); ?>
					<?php } ?>
					<div id="unmeasurable-table-progbar"></div>
					<div class="table-js" id="unmeasurable-table"></div>
				</div>
			</div>
		</div>
		<?php
	}

	/**
	 * QAHMで使いやすい形のリストを作成
	 */
	public function create_heatmap_list() {
		global $qahm_db;
		$list = array();

		// ホーム
		$list[] = array(
			'page_id'       => null,
			'version_index' => null,
			'url'           => esc_url( get_home_url() ),
			'title'         => esc_html__( 'Home', 'qa-heatmap-analytics' ),
			'id'            => 1,
			'type'          => 'home',
			'measure'       => 0,
			'change'        => false,
		);

		// 投稿、固定、カスタム投稿
		/*
		$posts = get_posts(
			array(
				'posts_per_page' => -1,
				'post_type'      => 'any',
			)
		);
		*/

		$table_name = $qahm_db->prefix . 'posts';
		$in_search_post_types = get_post_types( array( 'exclude_from_search' => false ) );
		$where    = " WHERE post_status = 'publish' AND post_type IN ('" . implode( "', '", array_map( 'esc_sql', $in_search_post_types ) ) . "')";
		$order    = ' ORDER BY post_date DESC';
		$query    = 'SELECT ID,post_title,post_type FROM ' . $table_name . $where . $order;
		$post_ary = $qahm_db->get_results( $query, ARRAY_A );


		// タイプ取得追加
		/*
		$add_post_type = $this->wrap_get_option( 'add_post_type' );
		if ( $this->wrap_get_option( 'add_post_type' ) ) {
			$add_post_type = $this->wrap_unserialize( $add_post_type );
			$add_posts = get_posts(
				array(
					'posts_per_page' => -1,
					'post_type'      => $add_post_type,
				)
			);
			$post_ary = array_merge( $post_ary, $add_posts );
		}
		*/
		$add_post_type = $this->wrap_get_option( 'add_post_type' );
		if ( $add_post_type ) {
			$in_search_post_types = $this->wrap_unserialize( $add_post_type );
			$where    = " WHERE post_status = 'publish' AND post_type IN ('" . implode( "', '", array_map( 'esc_sql', $in_search_post_types ) ) . "')";
			$order    = ' ORDER BY post_date DESC';
			$query    = 'SELECT ID,post_title,post_type FROM ' . $table_name . $where . $order;		$add_post_ary = $qahm_db->get_results( $query, ARRAY_A );
			$post_ary = array_merge( $post_ary, $add_post_ary );
		}


		foreach ( $post_ary as $post ) {
			switch ( $post['post_type'] ) {
				case 'post':
					$type = 'p';
					break;
				case 'page':
					$type = 'page_id';
					break;
				// default: $type = 'カスタム投稿'; break;
				default:
					$type = 'p';
					break;
			}
			$list[] = array(
				'page_id'       => null,
				'version_index' => null,
				'url'           => esc_url( get_permalink( $post['ID'] ) ),
				'title'         => esc_html( $post['post_title'] ),
				'id'            => (int) $post['ID'],
				'type'          => $type,
				//'slug'          => $post['post_name'],
				'measure'       => 0,
				'change'        => false,
			);
		}

		// カテゴリ、タグ、カスタムタクソノミー
		$terms = get_terms( array( 'get' => 'all' ) );
		$table_name_term = $qahm_db->prefix . 'terms';
		$table_name_tax  = $qahm_db->prefix . 'term_taxonomy';
		$order    = ' ORDER BY t.name ASC';
		$query    = 'SELECT t.term_id,t.name,tt.taxonomy FROM ' . $table_name_term . ' AS t INNER JOIN ' . $table_name_tax . ' AS tt ON t.term_id = tt.term_id' . $order;
		$term_ary = $qahm_db->get_results( $query, ARRAY_A );

		foreach ( $term_ary as $term ) {
			switch ( $term['taxonomy'] ) {
				case 'category':
					$type = 'cat';
					break;
				case 'post_tag':
					$type = 'tag';
					break;
				default:
					$type = 'tax';
					break;
			}
			$list[] = array(
				'page_id'       => null,
				'version_index' => null,
				'url'           => esc_url( get_term_link( (int) $term['term_id'] ) ),
				'title'         => esc_html( $term['name'] ),
				'id'            => (int) $term['term_id'],
				'type'          => $type,
				'measure'       => 0,
				'change'        => false,
			);
		}

		return $list;
	}

	/**
	 * テーブルの存在確認
	 * 他でも使うようならqahm baseに持っていっても良い
	 */
	private function exists_database_table( $table_name ) {
		global $wpdb;
		$query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $table_name ) );
		return $wpdb->get_var( $query ) === $table_name ? true : false;
	}

	/**
	 * qaテーブルからバージョン情報を作成
	 * cronでcase分けをするためcreate_heatmap_listとは関数を分けている
	 */
	public function add_version_info( $list ) {
		global $qahm_db;
		global $qahm_time;
		global $wp_filesystem;
		global $qahm_log;

		// テーブルの存在確認。cronによるdbテーブル作成処理がまだ実行されていない時用
		if ( ! $this->exists_database_table( $qahm_db->prefix . 'qa_pages' ) ||
			 ! $this->exists_database_table( $qahm_db->prefix . 'qa_page_version_hist' ) ||
			 ! $this->exists_database_table( $qahm_db->prefix . 'qa_pv_log' ) ) {
			return $list;
		}

		$cache_dir_path = $this->get_data_dir_path() . 'cache/';
		if ( ! $wp_filesystem->exists( $cache_dir_path ) ) {
			$wp_filesystem->mkdir( $cache_dir_path );
		}
		
		$list_temp_path     = $cache_dir_path . 'heatmap_list_temp.php';
		$list_idx_temp_path = $cache_dir_path . 'heatmap_list_idx_temp.php';
		$list_idx_def       = $this->wrap_get_contents( $list_idx_temp_path );
		if ( $list_idx_def ) {
			$list_idx_def = (int)$list_idx_def; 
		} else {
			$list_idx_def = 0;
		}

		for ( $list_idx = $list_idx_def, $list_max = count( $list ); $list_idx < $list_max; $list_idx++ ) {
			// 進行状況を定期的に保存
			if ( $list_idx % 10 === 0 && $list_idx !== $list_idx_def ) {
				$this->wrap_put_contents( $list_temp_path, $this->wrap_serialize( $list ) );
				$this->wrap_put_contents( $list_idx_temp_path, $list_idx );
			}
			
			/*
			// スラッシュあり、なしの両方を検索
			$url1 = $list[$list_idx]['url'];
			if( substr( $url1, -1 ) === '/' ) {
				$url2 = rtrim( $url1, '/' );
			} else {
				$url2 = $url1 . '/';
			}
            $url1_hash = hash( 'fnv164', $url1 );
            $url2_hash = hash( 'fnv164', $url2 );

			$table_name   = $qahm_db->prefix . 'qa_pages';
			$query        = 'SELECT page_id,url FROM ' . $table_name . ' WHERE url_hash IN(%s,%s)';
			$qa_pages_ary = $qahm_db->get_results( $qahm_db->prepare( $query, $url1_hash, $url2_hash ), ARRAY_A );
			if( ! $qa_pages_ary ) {
				continue;
			}
			$qa_pages = $qa_pages_ary[0];
			*/

			// wp_qa_idとwp_qa_typeで検索をかけ、最大値を取得
			$wp_qa_id   = $list[$list_idx]['id'];
			$wp_qa_type = $list[$list_idx]['type'];

			$table_name   = $qahm_db->prefix . 'qa_pages';
			$query        = 'SELECT page_id,url FROM ' . $table_name . ' WHERE wp_qa_id = %d AND wp_qa_type = %s';
			$qa_pages_ary = $qahm_db->get_results( $qahm_db->prepare( $query, $wp_qa_id, $wp_qa_type ), ARRAY_A );
			if( ! $qa_pages_ary ) {
				continue;
			}

			$table_name = $qahm_db->prefix . 'view_pv';
			$query      = 'SELECT count(*) FROM ' . $table_name . ' WHERE page_id = %d';
			$pv_max_idx = 0;
			$pv_max_num = 0;
			for ( $qa_pages_idx = 0, $qa_pages_max = count( $qa_pages_ary ); $qa_pages_idx < $qa_pages_max; $qa_pages_idx++ ) {
				$count = $qahm_db->get_results( $qahm_db->prepare( $query, $qa_pages_ary[$qa_pages_idx]['page_id'] ), ARRAY_A );
				if ( $pv_max_num < $count ) {
					$pv_max_idx = $qa_pages_idx;
					$pv_max_num = $count;
				}
			}
			$qa_pages = $qa_pages_ary[$pv_max_idx];
			$list[$list_idx]['page_id'] = (int) $qa_pages['page_id'];

			$table_name               = $qahm_db->prefix . 'view_page_version_hist';
			$query                    = 'SELECT page_id,version_id,device_id,version_no,insert_datetime FROM ' . $table_name . ' WHERE page_id = %d';
			$qa_page_version_hist_ary = $qahm_db->get_results( $qahm_db->prepare( $query, $qa_pages['page_id'] ), ARRAY_A );
			if ( ! $qa_page_version_hist_ary  ) {
				continue;
			}

			// version_noが大きい順にソート
			$ver_sort_ary = array();
			foreach ( $qa_page_version_hist_ary as $key => $value ) {
				$ver_sort_ary[$key] = (int) $value['version_no'];
			}
			array_multisort( $ver_sort_ary, SORT_DESC, $qa_page_version_hist_ary );


			// 最大のバージョンナンバーを求める（ソートされているのでインデックス0は最大値）
			$ver_no_max = (int) $qa_page_version_hist_ary[0]['version_no'];

			$ver_no_now          = null;
			$ins_date_unixtime   = null;
			$ins_date            = null;
			$period_to           = null;
			$period_from         = null;

			foreach ( $qa_page_version_hist_ary as $qa_page_version_hist_idx => $qa_page_version_hist ) {
				// 初回検索時
				if ( $qa_page_version_hist_idx === 0 ) {
					$ver_no_now                       = (int) $qa_page_version_hist['version_no'];
					$ins_date_unixtime                = $qahm_time->str_to_unixtime( $qa_page_version_hist['insert_datetime'] );
					$ins_date                         = $qahm_time->unixtime_to_str( $ins_date_unixtime, 'Y-m-d' );
					$period_to                        = $qahm_time->today_str();
					$period_from                      = $ins_date;
					$list[$list_idx]['version_index'] = $ver_no_now;
				}

				// version_infoの初期化
				if ( empty( $list[$list_idx]['version_info'][$qa_page_version_hist['version_no']] ) ) {
					$list[$list_idx]['version_info'][$qa_page_version_hist['version_no']] = array();
					$version_info = &$list[$list_idx]['version_info'][$qa_page_version_hist['version_no']];

					$version_info['data_all'] = 0;
					foreach ( QAHM_DEVICES as $qahm_dev ) {
						$version_info['data_' . $qahm_dev['name']] = 0;
						$version_info['version_id_' . $qahm_dev['name']] = null;
					}
				} else {
					$version_info = &$list[$list_idx]['version_info'][$qa_page_version_hist['version_no']];
				}

				// version_infoの設定
				$data_num = 0;
				$ins_date_unixtime = $qahm_time->str_to_unixtime( $qa_page_version_hist['insert_datetime'] );
				$ins_date          = $qahm_time->unixtime_to_str( $ins_date_unixtime, 'Y-m-d' );
				if ( $ver_no_now > (int) $qa_page_version_hist['version_no']) {
					$ver_no_now  = (int) $qa_page_version_hist['version_no'];
					$period_to   = $period_from;
					$period_from = $ins_date;
				}

				// 最新バージョンの検索時のみ、データ数を見る（view_pvを検索する）
				if ( $ver_no_max === $ver_no_now ) {
					$table_name    = $qahm_db->prefix . 'view_pv';
					$query         = 'SELECT access_time,is_raw_p,is_raw_e FROM ' . $table_name . ' WHERE version_id = %d';
					//$table_name    = $qahm_db->prefix . 'qa_pv_log';
					//$query         = 'SELECT access_time,raw_p,raw_e FROM ' . $table_name . ' WHERE version_id = %d';
					$qa_pv_log_ary = $qahm_db->get_results( $qahm_db->prepare( $query, $qa_page_version_hist['version_id'] ), ARRAY_A );

					if ( $qa_pv_log_ary ) {
						$fetch_now_utime = $qahm_time->now_unixtime();
						$older_utime     = $fetch_now_utime;
						foreach ( $qa_pv_log_ary as $qa_pv_log ) {
							if ( $qa_pv_log['is_raw_p'] || $qa_pv_log['is_raw_e'] ) {
							//if ( $qa_pv_log['raw_p'] || $qa_pv_log['raw_e'] ) {
								$data_num++;
								$access_utime = $qahm_time->str_to_unixtime( $qa_pv_log['access_time'] );
								if ( $access_utime < $older_utime ) {
									$older_utime = $access_utime;
								}
							}
						}
						if ( $older_utime !== $fetch_now_utime ) {
						$period_from = $qahm_time->unixtime_to_str( $older_utime, 'Y-m-d' );
						}
					}
				}

				if ( empty( $version_info['period_from'] ) ) {
					$version_info['period_from'] = $period_from;
					$version_info['period_to']   = $period_to;
				}

				$dev_name = $this->device_id_to_device_name( $qa_page_version_hist['device_id'] );
				$version_info[ 'version_id_' . $dev_name ] = (int) $qa_page_version_hist['version_id'];
				if ( $data_num > 0 ) {
					$version_info[ 'data_' . $dev_name ] = $data_num;
					$version_info[ 'data_all' ] += $data_num;
				}
			}
		}

		return $list;
	}

	/**
	 * 計測中の情報をリストに付与
	 */
	public function add_refresh_info( $heatmap_list ) {
		$cache_dir_path = $this->get_data_dir_path() . 'cache/';
		if ( ! $this->wrap_exists( $cache_dir_path ) ) {
			$this->wrap_mkdir( $cache_dir_path );
		}

		$refresh_path = $cache_dir_path . 'heatmap_list_refresh.php';
		if ( $this->wrap_exists( $refresh_path ) ) {
			$refresh_list = $this->wrap_unserialize( $this->wrap_get_contents( $refresh_path ) );

			foreach ( $refresh_list as $refresh ) {
				foreach( $heatmap_list as &$heatmap ) {
					if( $heatmap['page_id'] === $refresh['page_id'] ) {
						$heatmap['version_refresh'] = $refresh['version_no'];
						break;
					}
				}
			}

		}

		return $heatmap_list;
	}

	/**
	 * 計測中情報を削除
	 */
	public function delete_refresh_info() {
		$cache_dir_path = $this->get_data_dir_path() . 'cache/';
		if ( ! $this->wrap_exists( $cache_dir_path ) ) {
			$this->wrap_mkdir( $cache_dir_path );
		}

		$refresh_path = $cache_dir_path . 'heatmap_list_refresh.php';
		$this->wrap_delete( $refresh_path );
	}

	/**
	 * 計測フラグをリストに付与
	 */
	public function add_rec_flag( $list ) {
		global $qahm_recterm;
		$rt_recs = $qahm_recterm->get_all_record();
		if ( is_array( $list ) ) {
			for ( $i = 0; $i < count( $list ); $i++ ) {
				if ( $rt_recs ) {
					foreach ( $rt_recs as $rt_rec ) {
						if ( $list[ $i ]['type'] === $rt_rec['type'] && $list[ $i ]['id'] === (int) $rt_rec['ids'] ) {
							$list[ $i ]['measure'] = (int) $rt_rec['rec_flag'];
							break;
						}
						//usleep( 100 );
					}
				}
			}
		}

		return $list;
	}

	/**
	 * 設定画面の項目をデータベースに保存する
	 */
	public function save_config() {
		$post = array();
		// nonceで設定したcredentialのチェック

		// ヒートマップ管理画面
		if ( isset( $_POST[ self::NONCE_NAME ] ) && $_POST[ self::NONCE_NAME ] ) {
			if ( check_admin_referer( self::NONCE_ACTION, self::NONCE_NAME ) ) {
				//$this->wrap_update_option( 'is_first_heatmap_setting', false );
				// falseだとkeyがdbにない場合、値が何も入らないので確実に空欄が1入るようにしている
				$this->wrap_update_option( 'is_first_heatmap_setting', '' );

				/*
				// ライセンス認証による制限 あとで今の仕様に変更
				if ( ! $this->wrap_get_option( 'license_auth' ) ) {
					$post['list_rec_num'] = $this->wrap_filter_input( INPUT_POST, 'list_rec_num' );
					if ( $post['list_rec_num'] > QAHM_License::FREE_MEASURE_PAGE ) {
						self::$error_msg['list_rec_num'] = ( QAHM_License::FREE_MEASURE_PAGE + 1 ) . 'ページ以上の計測を行う場合はライセンス認証を行ってください。';
						return;
					}
				}
				*/

				global $wpdb;
				global $qahm_time;
				global $qahm_recterm;
				$db_rts   = $qahm_recterm->get_all_record();
				$bulk_rts = array();

				$post['list_num'] = $this->wrap_filter_input( INPUT_POST, 'list_num' );
				if ( $post['list_num'] > 0 ) {
					for ( $i = 0; $i < $post['list_num']; $i++ ) {
						$post[ 'type_' . $i ]     = $this->wrap_filter_input( INPUT_POST, 'type_' . $i );
						$post[ 'ids_' . $i ]      = $this->wrap_filter_input( INPUT_POST, 'ids_' . $i );
						$post[ 'rec_days_' . $i ] = $this->wrap_filter_input( INPUT_POST, 'rec_days_' . $i );
						$post[ 'rec_flag_' . $i ] = $this->wrap_filter_input( INPUT_POST, 'rec_flag_' . $i );
						if ( ! $post[ 'type_' . $i ] || ! $post[ 'ids_' . $i ] ) {
							continue;
						}

						$idx_rt = -1;

						if ( $db_rts ) {
							$db_rts_cnt = count( $db_rts );
							for ( $j = 0; $j < $db_rts_cnt; $j++ ) {
								if ( $post[ 'type_' . $i ] == $db_rts[ $j ]['type'] &&
									$post[ 'ids_' . $i ] == $db_rts[ $j ]['ids'] ) {
									$idx_rt = $j;
									break;
								}
								usleep( 100 );
							}
						}

						// recterm内に見つからなかった場合、recrefreshも無いということなのでrecrefreshも作成する
						if ( $idx_rt === -1 ) {
							$bulk_rts[] = array(
								'type'        => $post[ 'type_' . $i ],
								'ids'         => $post[ 'ids_' . $i ],
								'rec_flag'    => $post[ 'rec_flag_' . $i ],
								'update_date' => $post[ 'rec_flag_' . $i ] ? date_i18n( 'Y-m-d' ) : null,
							);

						} else {
							// rec_flagを前回オフからオンに切り替えた時はupdate_dateを更新
							if ( $post[ 'rec_flag_' . $i ] && ! $db_rts[ $idx_rt ]['rec_flag'] ) {
								$update_date = date_i18n( 'Y-m-d' );
							} else {
								$update_date = $db_rts[ $idx_rt ]['update_date'];
							}

							$bulk_rts[] = array(
								'type'        => $db_rts[ $idx_rt ]['type'],
								'ids'         => $db_rts[ $idx_rt ]['ids'],
								'rec_days'    => $post[ 'rec_days_' . $i ] ? $post[ 'rec_days_' . $i ] : $db_rts[ $idx_rt ]['rec_days'],
								'rec_flag'    => $post[ 'rec_flag_' . $i ],
								'file_size'   => $db_rts[ $idx_rt ]['file_size'],
								'insert_date' => $db_rts[ $idx_rt ]['insert_date'],
								'update_date' => $update_date,
								'start_date'  => $db_rts[ $idx_rt ]['start_date'],
								'stop_date'   => $db_rts[ $idx_rt ]['stop_date'],
							);
						}

						usleep( 100 );
					}

					// バルクインサート
					if ( isset( $bulk_rts ) ) {
						$qahm_recterm->bulk_insert( $bulk_rts );
					}
				}
			}
		}
	}

	/**
	 * ヒートマップリストを取得
	 */
	public function ajax_create_heatmap_list() {
		global $wp_filesystem;

		$cache_dir_path = $this->get_data_dir_path() . 'cache/';
		$this->wrap_mkdir( $cache_dir_path );
		
		// リストファイル読み込み
		$list_path      = $cache_dir_path . 'heatmap_list.php';
		$list_temp_path = $cache_dir_path . 'heatmap_list_temp.php';

		if ( $wp_filesystem->exists( $list_path ) ) {
			$list = $this->wrap_unserialize( $this->wrap_get_contents( $list_path ) );
		} else {
			// 一時的なリスト作成
			if ( $wp_filesystem->exists( $list_temp_path ) ) {
				$list = $this->wrap_unserialize( $this->wrap_get_contents( $list_temp_path ) );
			} else {
				$list = $this->create_heatmap_list();
				$this->wrap_put_contents( $list_temp_path, $this->wrap_serialize( $list ) );
			}

			// 一時的なリストにバージョン情報を追加してリストを完成させる
			$list = $this->add_version_info( $list );
		
			// リスト配列の中身をファイルに保存
			$this->wrap_put_contents( $list_path, $this->wrap_serialize( $list ) );
		}

		$list = $this->add_refresh_info( $list );
		$list = $this->add_rec_flag( $list );

		echo wp_json_encode( $list );
		die();
	}

	/**
	 * バージョン切り替え
	 */
	public function ajax_refresh_version() {
		$nonce = $this->wrap_filter_input( INPUT_POST, 'nonce' );
		if ( ! wp_verify_nonce( $nonce, self::NONCE_REFRESH ) ) {
			http_response_code( 400 );
			die( 'wp_verify_nonce error' ); 
		}

		global $wp_filesystem;
		global $wpdb;
		global $qahm_time;
		$today_str  = $qahm_time->today_str();
		$now_str    = $qahm_time->now_str();

		$page_id    = (int) $this->wrap_filter_input( INPUT_POST, 'page_id' );
		$wp_qa_type = $this->wrap_filter_input( INPUT_POST, 'wp_qa_type' );
		$wp_qa_id   = (int) $this->wrap_filter_input( INPUT_POST, 'wp_qa_id' );
		$url        = $this->wrap_filter_input( INPUT_POST, 'url' );
		$title      = $this->wrap_filter_input( INPUT_POST, 'title' );

		if ( empty( $page_id ) ) {
			// qa_pages insert
			$table_name = $wpdb->prefix . 'qa_pages';
			
			$wpdb->insert( 
				$table_name, 
				array( 
					'tracking_id' => $this->get_tracking_id(),
					'wp_qa_type'  => $wp_qa_type, 
					'wp_qa_id'    => $wp_qa_id,
					'url'         => $url,
					'url_hash'    => hash( 'fnv164', $url ),
					'title'       => $title,
					'update_date' => $today_str
				), 
				array( 
					'%s',
					'%s',
					'%d',
					'%s',
					'%s',
					'%s',
					'%s'
				) 
			);
			$page_id = $wpdb->insert_id;
		}
		
		$ver_hist_bulk_ary         = array();
		$ver_hist_place_holder_ary = array();

		// 最新バージョンを調べる。存在しなければバージョン1作成
		// 存在していれば全デバイスそのバージョンを作成＆新規バージョン追加
		$table_name           = $wpdb->prefix . 'qa_page_version_hist';
		$query                = 'SELECT version_id,device_id,version_no FROM ' . $table_name . ' WHERE page_id=%d and version_no=(select max(version_no) FROM ' . $table_name . ' WHERE page_id=%d)';
		$qa_page_version_hist = $wpdb->get_results( $wpdb->prepare( $query, $page_id, $page_id ) );

		$dev_id_ary = array();
		$cur_ver_no = 0;

		if ( $qa_page_version_hist ) {
			foreach ( $qa_page_version_hist as $hist ) {
				$dev_id_ary[] = $hist->device_id;
				$cur_ver_no   = $hist->version_no;
			}
		}

		// 今のバージョンは全デバイス分データを作成
		foreach ( QAHM_DEVICES as $qahm_dev ) {
			$dev_name  = $this->device_id_to_device_name( $qahm_dev['id'] );
			
			// デバイスによるユーザーエージェント指定
			$response = $this->wrap_remote_get( $url, $dev_name );
			if ( is_wp_error( $response ) ) {
				http_response_code( 400 );
				die( 'wp_remote_get is wp error' ); 
			} elseif( $response['response']['code'] !== 200 ) {
				http_response_code( 400 );
				die( 'wp_remote_get status code is other than 200' ); 
			} else {
				$base_html = $response['body'];
			}

			if ( $qa_page_version_hist ) {
				$find = false;
				if ( $dev_id_ary !== array() ) {
					foreach ( $dev_id_ary as $dev_id ) {
						if ( $qahm_dev['id'] === (int) $dev_id ) {
							$find = true;
							break;
						}
					}
				}

				// 現行バージョンで作られていないデバイスのversion histを作成
				if ( ! $find ) {
					$ver_hist_bulk_ary[]         = $page_id;
					$ver_hist_bulk_ary[]         = $qahm_dev['id'];
					$ver_hist_bulk_ary[]         = $cur_ver_no;
					$ver_hist_bulk_ary[]         = $base_html;
					$ver_hist_bulk_ary[]         = $today_str;
					$ver_hist_bulk_ary[]         = $now_str;
					$ver_hist_place_holder_ary[] = '(%d, %d, %d, %s, %s, %s)';
				}
			}

			// バージョン追加
			$ver_hist_bulk_ary[] = $page_id;
			$ver_hist_bulk_ary[] = $qahm_dev['id'];
			$ver_hist_bulk_ary[] = $cur_ver_no + 1;
			$ver_hist_bulk_ary[] = $base_html;
			$ver_hist_bulk_ary[] = $today_str;
			$ver_hist_bulk_ary[] = $now_str;
			$ver_hist_place_holder_ary[] = '(%d, %d, %d, %s, %s, %s)';
		}

		// バルクインサート
		$query = 'INSERT INTO ' . $table_name . ' ' .
				'(page_id, device_id, version_no, base_html, update_date, insert_datetime) ' .
				'VALUES ' . join( ',', $ver_hist_place_holder_ary ) . ' ' .
				'ON DUPLICATE KEY UPDATE ' .
				'page_id = VALUES(page_id), ' .
				'device_id = VALUES(device_id), ' .
				'version_no = VALUES(version_no), ' .
				'base_html = VALUES(base_html), ' .
				'update_date = VALUES(update_date), ' .
				'insert_datetime = VALUES(insert_datetime)';
		$data['result']  = $wpdb->query( $wpdb->prepare( $query, $ver_hist_bulk_ary ) );
		$data['version'] = $cur_ver_no + 1;

		$data['version_switch_success_text'] = sprintf( esc_html__( 'Switched to version %1$d. %2$s The data will be available tomorrow or later.', 'qa-heatmap-analytics' ), $data['version'], '<br>' );

		
		// その日のバージョンアップファイルを作成
		$cache_dir_path = $this->get_data_dir_path() . 'cache/';
		if ( ! $wp_filesystem->exists( $cache_dir_path ) ) {
			$wp_filesystem->mkdir( $cache_dir_path );
		}

		$refresh_path = $cache_dir_path . 'heatmap_list_refresh.php';
		$find  = false;
		$index = 0;
		if ( $wp_filesystem->exists( $refresh_path ) ) {
			$refresh_list = $this->wrap_unserialize( $this->wrap_get_contents( $refresh_path ) );

			foreach ( $refresh_list as &$refresh ) {
				if( $refresh['page_id'] === $page_id ) {
					$refresh['version_no'] = $data['version'];
					$find                = true;
					break;
				}
				$index++;
			}

		} else {
			$refresh_list = array();
		}

		if ( ! $find ) {
			$refresh_list[ $index ] = array(
				'page_id'    => $page_id,
				'version_no' => $data['version']
			);
		}

		$this->wrap_put_contents( $refresh_path, $this->wrap_serialize( $refresh_list ) );

		echo wp_json_encode( $data );
		die();
	}

	/**
	 * 実績の設定
	 */
	public function ajax_set_achievements() {
		$achievements = json_decode( $this->wrap_filter_input( INPUT_POST, 'achievements' ), true );
		$this->wrap_update_option( 'achievements', $achievements );
	}

	/**
	 * 全ページ計測チェックの変更
	 */
	public function ajax_change_is_raw_save_all() {
		$is_raw_save_all = $this->wrap_get_option( 'is_raw_save_all' );
		if ( $is_raw_save_all ) {
			$is_raw_save_all = '';
		} else {
			$is_raw_save_all = 'on';
		}
		
		$this->wrap_update_option( 'is_raw_save_all', $is_raw_save_all );
	}

	/**
	 * バージョンIDから全データ数を返す
	 */
	public function ajax_get_data_num() {
		try {
			global $qahm_db;

			// $ver_id_aryは全端末のversion_idが入っている配列
			$ver_id_ary = json_decode( $this->wrap_filter_input( INPUT_POST, 'version_id_ary' ), true );
			
			$table_name   = $qahm_db->prefix . 'view_pv';
			$query        = 'SELECT is_raw_p,is_raw_e FROM ' . $table_name . ' WHERE version_id = %d';
			//$table_name   = $qahm_db->prefix . 'qa_pv_log';
			//$query        = 'SELECT raw_p,raw_e FROM ' . $table_name . ' WHERE version_id = %d';
			$data_num_ary = array();
			
			foreach ( $ver_id_ary as $ver_idx => $ver_id ) {
				if ( ! $ver_id ) {
					continue;
				}

				$data_num_ary[$ver_idx] = 0;
				$qa_pv_log_ary = $qahm_db->get_results( $qahm_db->prepare( $query, $ver_id ), ARRAY_A );
				if ( ! $qa_pv_log_ary ) {
					continue;
				}

				foreach ( $qa_pv_log_ary as $qa_pv_log ) {
					if( $qa_pv_log['is_raw_p'] || $qa_pv_log['is_raw_e'] ) {
					//if( $qa_pv_log['raw_p'] || $qa_pv_log['raw_e'] ) {
						$data_num_ary[$ver_idx]++;
					}
				}
			}

			echo wp_json_encode( $data_num_ary );
		} catch ( Exception $ex ) {
			echo '';
		}
		die();
	}


	/**
	 * 計測不可となっている投稿タイプの記事一覧配列を作成
	 */
	public function ajax_create_unmeasurable_table() {
		global $qahm_db;

		// wp_postsから記事一覧を取得
		$table_name = $qahm_db->prefix . 'posts';
		$query      = 'SELECT ID, post_title, post_type FROM ' . $table_name . ' WHERE post_status = "publish"';
		$wp_posts   = $qahm_db->get_results( $query );
		
		// リストファイル読み込み
		$cache_dir_path    = $this->get_data_dir_path() . 'cache/';
		$list_path = $cache_dir_path . 'heatmap_list.php';
		$list_ary  = null;
		if ( $this->wrap_exists( $list_path ) ) {
			$list_ary  = $this->wrap_unserialize( $this->wrap_get_contents( $list_path ) );
		} else {
			echo wp_json_encode( null );
			die();
		}

		/*
		// 除外対象の投稿タイプ配列を作成
		$exclude_type_ary = array();
		foreach ( $list_ary as $list_val ) {
			if ( ! in_array( $list_val['type'], $exclude_type_ary ) ) {
				$exclude_type_ary[] = $list_val['type'];
			}
		}

		$post_ary = array();
		foreach ( $wp_posts as $wp_post ) {
			//if ( in_array( $wp_post->post_type, $exclude_type_ary ) ) {
			//	continue;
			//}
			$exclude_post_find = false;
			foreach ( $list_ary as $list_val ) {
				if ( $wp_post->post_title === $list_val['title'] || in_array( $wp_post->post_type, $exclude_type_ary ) ) {
					$exclude_post_find = true;
					break;
				}
			}
			if ( $exclude_post_find ) {
				continue;
			}

			$post_ary[] = array(
				'post_url'   => get_permalink( $wp_post->ID ),
				'post_title' => $wp_post->post_title,
				'post_type'  => $wp_post->post_type,
			);
		}
		*/

		// 除外対象の投稿タイプ配列を作成
		// ヒートマップリストのキャッシュでは元々の投稿タイプの情報が抜け落ちているので、ここではget_postsで除外する投稿タイプを取得する
		$exclude_type_ary = array(
			'post',
			'page',
			'attachment',
			'revision',
			'nav_menu_item',
			'custom_css',
			'customize_changeset',
			'wp_block',
			'wp_template',
			'wp_template_part',
			'wp_global_styles',
			'wp_navigation',
		);
		/*
		$exclude_post_ary = get_posts(
			array(
				'posts_per_page' => -1,
				'post_type'      => 'any',
			)
		);*/
		
		$table_name = $qahm_db->prefix . 'posts';
		$in_search_post_types = get_post_types( array( 'exclude_from_search' => false ) );
		$where = " WHERE post_status = 'publish' AND post_type IN ('" . implode( "', '", array_map( 'esc_sql', $in_search_post_types ) ) . "')";
		$order = ' ORDER BY post_date DESC';
		$query = 'SELECT ID,post_title,post_type FROM ' . $table_name . $where . $order;
		$exclude_post_ary = $qahm_db->get_results( $query, ARRAY_A );

		foreach ( $exclude_post_ary as $exclude_post ) {
			if ( ! in_array( $exclude_post['post_type'], $exclude_type_ary ) ) {
				$exclude_type_ary[] = $exclude_post['post_type'];
			}
		}

		$post_ary = array();
		foreach ( $wp_posts as $wp_post ) {
			// 除外対象か確認
			if ( in_array( $wp_post->post_type, $exclude_type_ary ) || $wp_post->post_title === '' ) {
				continue;
			}

			// ヒートマップ記事一覧と被りのタイトルがあるか確認
			$exclude_find = false;
			foreach ( $list_ary as $list_val ) {
				if ( $wp_post->post_title === $list_val['title'] ) {
					$exclude_find = true;
					break;
				}
			}
			if ( $exclude_find ) {
				continue;
			}

			// 除外対象じゃない場合、計測不可リストに追加
			$post_ary[] = array(
				'post_url'   => get_permalink( $wp_post->ID ),
				'post_title' => esc_html( $wp_post->post_title ),
				'post_type'  => esc_html( $wp_post->post_type ),
			);
		}
		if ( ! $post_ary ) {
			echo wp_json_encode( null );
			die();
		}

		echo wp_json_encode( $post_ary );
		die();
	}

} // end of class
