<?php
/**
 * 
 *
 * @package qa_heatmap
 */

$qahm_admin_page_help = new QAHM_Admin_Page_Help();

class QAHM_Admin_Page_Help extends QAHM_Admin_Page_Base {

	// スラッグ
	const SLUG = QAHM_NAME . '-help';

	// nonce
	const NONCE_ACTION = self::SLUG . '-nonce-action';
	const NONCE_NAME   = self::SLUG . '-nonce-name';

	/**
	 * コンストラクタ
	 */
	public function __construct() {
		parent::__construct();
	}
	
	/**
	 * 初期化
	 */
	public function enqueue_scripts( $hook_suffix ) {
		if( $this->hook_suffix !== $hook_suffix ||
			! $this->is_enqueue_jquery()
		) {
			return;
		}

		$css_dir_url = $this->get_css_dir_url();

		// enqueue_style
		$this->common_enqueue_style();
		wp_enqueue_style( QAHM_NAME . '-admin-page-help-css', $css_dir_url . 'admin-page-help.css', array( QAHM_NAME . '-reset' ), QAHM_PLUGIN_VERSION );	


		// enqueue script
		$this->common_enqueue_script();
	
		// inline script
		$scripts = $this->get_common_inline_script();
		wp_add_inline_script( QAHM_NAME . '-common', 'var ' . QAHM_NAME . ' = ' . QAHM_NAME . ' || {}; let ' . QAHM_NAME . 'Obj = ' . wp_json_encode( $scripts ) . '; ' . QAHM_NAME . ' = Object.assign( ' . QAHM_NAME . ', ' . QAHM_NAME . 'Obj );', 'before' );

		// localize
		$localize = $this->get_common_localize_script();
		wp_localize_script( QAHM_NAME . '-common', QAHM_NAME . 'l10n', $localize );
	}

	/**
	 * ページの表示
	 */
	public function create_html() {
		$lang_set = get_bloginfo('language');
		$php_version = phpversion();
		$php_memory_limit = ini_get( 'memory_limit' );
		$php_max_execution_time = ini_get( 'max_execution_time' ); 
		global $wp_version;
		?>

		<div id="<?php esc_attr_e( basename( __FILE__, '.php' ) ); ?>" class="qahm-admin-page">
			<div class="wrap">
				<h1>QA <?php esc_html_e( 'Help', 'qa-heatmap-analytics' ); ?></h1>
				<?php $this->view_announce_html(); ?>

				<div id="qa-help">

					<?php 
						if ( $lang_set !== 'ja' ) :
					?>
					<div class="english-note-important">
						<h4><?php esc_html_e( '[Please Note]', 'qa-heatmap-analytics' ); ?></h4>
						<p><?php esc_html_e( 'The language switcher on our support site may not always work properly. Please click the "language" button at the top right of the screen to switch the language.', 'qa-heatmap-analytics' ); ?>
						<br><?php esc_html_e( 'Some of the articles in our support site may be written only in Japanese. We would appreciate it if you could use a browser\'s translation service or another tool to read those articles.', 'qa-heatmap-analytics' ); ?></p>
					</div> 
					<?php endif; ?>

					<div class="help01">
						<h2><?php esc_html_e( 'Plugin and Site info', 'qa-heatmap-analytics' ); ?></h2>
						<h2 class="note-version"><?php esc_html_e( 'Your Installed Version of QA Analytics', 'qa-heatmap-analytics' ); ?>: <?php echo esc_html( QAHM_PLUGIN_VERSION ); ?></h2>
						<table>
							<thead>
								<tr>
									<th></th>
									<td class="yours"><?php esc_html_e( 'Your Site', 'qa-heatmap-analytics' ); ?></td><td class="qas">/ <?php esc_html_e( 'QA Supported Environment', 'qa-heatmap-analytics' ); ?></td>
								</tr>
							</thead>
							<tbody>
								<tr>
									<th><?php esc_html_e( 'WordPress version', 'qa-heatmap-analytics' ); ?></th>
									<td class="yours"><?php echo $wp_version; ?></td><td class="qas">/ <?php esc_html_e( 'At least 5.6', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php esc_html_e( 'PHP version', 'qa-heatmap-analytics' ); ?></th>
									<td class="yours"><?php echo $php_version; ?></td><td class="qas">/ <?php esc_html_e( '5.6 or higher', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php esc_html_e( 'PHP memory limit', 'qa-heatmap-analytics' ); ?></th>
									<td class="yours"><?php echo $php_memory_limit; ?></td><td class="qas">/ <?php esc_html_e( '1G+(1024M+) recommended', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php esc_html_e( 'PHP max execution time', 'qa-heatmap-analytics' ); ?></th>
									<td class="yours"><?php echo $php_max_execution_time; ?></td><td class="qas">/ <?php esc_html_e( '240 seconds recommended', 'qa-heatmap-analytics' ); ?></td>
								</tr>
							</tbody>
						</table>
						<p><a href="https://mem.quarka.org/manual/site-environment/" target="_blank" rel="noopener"><?php esc_html_e( 'Supported site environment', 'qa-heatmap-analytics' ); ?><span class="qahm_link-mark"><i class="fas fa-external-link-alt"></i></span></a></p>
					</div>

					<div class="h-frame help02">
						<h2><?php esc_html_e( 'User Guide', 'qa-heatmap-analytics' ); ?></h2>
						<h3><a href="https://mem.quarka.org/manual/" target="_blank" rel="noopener"><?php esc_html_e( 'Support Site / QA Manuals', 'qa-heatmap-analytics' ); ?></a></h3>
						<p><?php esc_html_e( 'Get information about how to use and set up QA Analytics, troubleshooting, etc.', 'qa-heatmap-analytics' ); ?></p>
					</div>

					<div class="h-frame help03">		
						<h2><?php esc_html_e( 'QA Support', 'qa-heatmap-analytics' ); ?></h2>
						<h3><a href="https://mem.quarka.org/memberhome/" target="_blank" rel="noopener"><?php esc_html_e( 'Contact our Support Team Directly (login needed)', 'qa-heatmap-analytics' ); ?></a></h3>
						<p><?php esc_html_e( 'Those who subscribe to QA options can use a direct-contact-form in our support site. Please log in and fill the form.', 'qa-heatmap-analytics' ); ?><br>
						<?php esc_html_e( 'The "Debug" information which will be needed for technical questions such as about plugin operations is at the bottom of this Help screen.', 'qa-heatmap-analytics' ); ?></p>
						<h3><a href="https://quarka.org/function-business-support/" target="_blank" rel="noopener"><?php esc_html_e( 'QA Support Option', 'qa-heatmap-analytics' ); ?></a></h3>
						<p><?php esc_html_e( 'If you need support with individual consultation for issues, optional support is available.', 'qa-heatmap-analytics' ); ?></p>
					</div>

					<div class="h-frame help03">		
						<h2><?php esc_html_e( 'WordPress Support Forum', 'qa-heatmap-analytics' ); ?></h2>
						<h3><a href="https://wordpress.org/support/plugin/qa-heatmap-analytics/" target="_blank" rel="noopener"><?php esc_html_e( 'QA Analytics << WordPress Support Forum', 'qa-heatmap-analytics' ); ?></a></h3>
						<p><?php esc_html_e( 'This forum is a public place offered by WordPress.org to have conversation about QA Analytics.', 'qa-heatmap-analytics' ); ?><br>
						<?php esc_html_e( 'If you have a question, you can ask about it there. Our support team can help you; as well as other kind people will. To let us know something you have noticed is also welcome.', 'qa-heatmap-analytics' ); ?><br>
						<?php esc_html_e( '(*You need to log in with your WordPress.org account to use the forum. If you do not have one, please create an account.)', 'qa-heatmap-analytics' ); ?></p>
					</div>

							
					<div class="h-frame help04">
						<h2><?php esc_html_e( 'Glossary', 'qa-heatmap-analytics' ); ?></h2>
						<p><?php esc_html_e( 'This briefly explains the terms that appear on the QA screens.', 'qa-heatmap-analytics' ); ?><br>
						<?php printf( esc_html__( 'For more details, see %1$s QA Manuals %2$s in our support site.', 'qa-heatmap-analytics' ), '<a href="https://mem.quarka.org/manual/" target="_blank" rel="noopener">', '</a>'); ?></p>
						<table>
							<tbody>
								<tr>
									<th><?php esc_html_e( 'Sessions (Earliest 7days)', 'qa-heatmap-analytics' ); ?></th>
									<td><?php esc_html_e( 'The total number of sessions for the 7 days starting from the oldest date in the date range.', 'qa-heatmap-analytics' ); ?><br>
									<?php esc_html_e( 'When the date range is 7 days or shorter, it applies the earliest one day to avoid having the same total value for earliest and latest.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php esc_html_e( 'Sessions (Latest 7days)', 'qa-heatmap-analytics' ); ?></th>
									<td><?php esc_html_e( 'The total number of sessions for the 7 days backward from the newest date in the date range.', 'qa-heatmap-analytics' ); ?><br>
									<?php esc_html_e( 'When the date range is 7 days or shorter, it applies the latest one day to avoid having the same total value for earliest and latest.', 'qa-heatmap-analytics' ); ?></td>
								</tr>								
								<tr>
									<th><?php esc_html_e( 'Average Time on Page', 'qa-heatmap-analytics' ); ?></th>
									<td><?php esc_html_e( 'Shows the average amount of time spent on the page per person, calcurated by: sum total of time on page / amount of data.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php esc_html_e( 'Time on Site', 'qa-heatmap-analytics' ); ?></th>
									<td><?php printf( esc_html__( 'The time while a user actually browses your site is measured with an accuracy of +-3 seconds; this is the point %1$s differs from Google Analytics %2$s.', 'qa-heatmap-analytics' ), '<i>', '</i>' ); ?><br>
									<?php esc_html_e( 'QA Analytics doesn\'t count time while a user is browsing another site or using other applications, so you can see an accurate time range that the user stayed.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php esc_html_e( 'Scroll Map', 'qa-heatmap-analytics' ); ?></th>
									<td><?php esc_html_e( 'Indicates "whether this page was read all the way down."', 'qa-heatmap-analytics' ); ?><br>
									<?php esc_html_e( 'It also shows the number of the users at the bottom.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php esc_html_e( 'Attention Map', 'qa-heatmap-analytics' ); ?></th>
									<td><?php esc_html_e( 'Indicates "what content the users are interested in."', 'qa-heatmap-analytics' ); ?><br>
									<?php esc_html_e( 'Well-read parts are displayed in red like a (temperature) heat map.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php esc_html_e( 'Click Heatmap', 'qa-heatmap-analytics' ); ?></th>
									<td><?php esc_html_e( 'Indicates "where the users are clicking."', 'qa-heatmap-analytics' ); ?><br>
									<?php esc_html_e( 'The place a user clicks is likely to be the point of interest. The parts that have been clicked a lot are displayed in red.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php esc_html_e( 'Click Count Map', 'qa-heatmap-analytics' ); ?></th>
									<td><?php esc_html_e( 'Indicates "how many times this button or link has been pressed."', 'qa-heatmap-analytics' ); ?><br>
									<?php esc_html_e( 'You can see trends such as banner clicks.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php esc_html_e( 'Replay', 'qa-heatmap-analytics' ); ?><br><?php esc_html_e( '(Session Replay)', 'qa-heatmap-analytics' ); ?></th>
									<td><?php esc_html_e( 'Session Replay is a function that shows you how the page was viewed in a session, likely to follow the user\'s line of sight.', 'qa-heatmap-analytics' ); ?><br>
									<?php esc_html_e( 'Press a "play" button of each session row in the list, which you can find at "Realtime View" or Sessions/Heatmap on "Home > Conversion", to replay.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php esc_html_e( 'Download TSV Data', 'qa-heatmap-analytics' ); ?></th>
									<td><?php esc_html_e( 'QA Analytics records every pageview*, and you can download those data!', 'qa-heatmap-analytics' ); ?><br>
									<?php esc_html_e( 'Download the file from "Overview" section for the whole data, or from "Filter and See" section for the data of a specific page you want.', 'qa-heatmap-analytics' ); ?><br>
									<?php esc_html_e( '*Pageview data include: os, browser, page url and title, device category, utm_source, source_domain, utm_medium, utm_campaign, session_number, access_time, ordinal of pv in a session, speed_msec, browse_sec, is_last pv (or not), is_new-user (or returning).', 'qa-heatmap-analytics' ); ?><br>
									<?php esc_html_e( '**TSV is a text format in which each value is separated with tabs.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
							</tbody>
						</table>
					</div>



				</div>

			</div>
		</div>

		
		
		
		<!-- Start debug -->
		<?php
			global $wpdb;
		
			$my_theme     = wp_get_theme();
			$site_plugins = get_plugins();
			$plugin_names = [];
		
			foreach( $site_plugins as $main_file => $plugin_meta ) {
				if ( ! is_plugin_active( $main_file ) ) {
					continue;
				}
				$plugin_names[] = sanitize_text_field( $plugin_meta['Name'] . ' ' . $plugin_meta['Version'] );
			}
			
			$options = '';
			/*
			foreach ( QAHM_OPTIONS as $key => $value ) {
				$options .= '<p><strong>' . $key . ':</strong><br>' . $this->wrap_get_option( $key ) . '</p>';
			}
			*/
			$options .= '<p><strong>Plugin version:</strong><br>' . QAHM_PLUGIN_VERSION . '</p>';

			$cron_status = $this->wrap_get_contents( $this->get_data_dir_path() . 'cron_status' );
		?>
		<div id="qahm-help-debug">
			<h3>Debug</h3>
			<hr>
			<p><strong>Server IP address:</strong><br><?php echo $this->wrap_filter_input( INPUT_SERVER, 'SERVER_ADDR' ); ?></p>
			<p><strong>PHP version:</strong><br><?php echo $php_version; ?></p>
			<p><strong>PHP memory limit:</strong><br><?php echo $php_memory_limit; ?></p>
			<p><strong>PHP extensions:</strong><br><?php echo implode( ', ', get_loaded_extensions() ); ?></p>
			<p><strong>Database version:</strong><br><?php echo $wpdb->get_var( "SELECT VERSION();" ); ?></p>
			<p><strong>InnoDB availability:</strong><br><?php echo $wpdb->get_var( "SELECT SUPPORT FROM INFORMATION_SCHEMA.ENGINES WHERE ENGINE = 'InnoDB';" ); ?></p>
			<p><strong>WordPress version:</strong><br><?php echo $wp_version; ?></p>
			<p><strong>Multisite:</strong><br><?php echo ( function_exists( 'is_multisite' ) && is_multisite() ) ? 'Yes' : 'No'; ?></p>
			<p><strong>Active plugins:</strong><br><?php echo implode( ', ', $plugin_names ); ?></p>
			<p><strong>Theme:</strong><br><?php echo $my_theme->get( 'Name' ) . ' (' . $my_theme->get('Version') . ') by ' . $my_theme->get('Author'); ?></p>
			<?php echo $options; ?>
			<p><strong>qalog.txt:</strong><br><?php echo $this->get_data_dir_url( 'log' ) . 'qalog.txt'; ?></p>
			<p><strong>cron_status:</strong><br><?php echo $cron_status; ?></p>
			<p><strong>max_execution_time:</strong><br><?php echo $php_max_execution_time; ?></p>
		</div>
		<!-- End debug -->

		<?php
	}
} // end of class
