<?php
/**
 * 
 *
 * @package qa_heatmap
 */

$qahm_admin_page_license = new QAHM_Admin_Page_License();

class QAHM_Admin_Page_License extends QAHM_Admin_Page_Base {

	// スラッグ
	const SLUG = QAHM_NAME . '-license';

	// nonce
	const NONCE_ACTION = self::SLUG . '-nonce-action';
	const NONCE_NAME   = self::SLUG . '-nonce-name';

	private static $error_msg = array();

	/**
	 * コンストラクタ
	 */
	public function __construct() {
		parent::__construct();
		add_action( 'admin_init', array( $this, 'save_config' ) );
		$this->regist_ajax_func( 'ajax_clear_license_message' );
	}

	/**
	 * 初期化
	 */
	public function enqueue_scripts( $hook_suffix ) {
		if( $this->hook_suffix !== $hook_suffix ||
			! $this->is_enqueue_jquery()
		) {
			return;
		}

		// enqueue_style
		$this->common_enqueue_style();
	
		// enqueue script
		$this->common_enqueue_script();
		wp_enqueue_script( QAHM_NAME . '-admin-page-license', plugins_url( 'js/admin-page-license.js', __FILE__ ), array( QAHM_NAME . '-effect' ), QAHM_PLUGIN_VERSION, false );
	
		// inline script
		$scripts = $this->get_common_inline_script();
		// 紙吹雪エフェクト。エフェクト実行後はajaxでupdateして紙吹雪フラグをオフにするのが適切だが、そこにかける時間ももったいないのでここでupdateしている
		$msg_ary = $this->wrap_get_option( 'license_message' );
		if ( ! empty( $msg_ary[0]['confetti'] ) ) {
			$scripts['license_confetti'] = $msg_ary[0]['confetti'];
			unset( $msg_ary[0]['confetti'] );
			$this->wrap_update_option( 'license_message', $msg_ary );
		}
		wp_add_inline_script( QAHM_NAME . '-common', 'var ' . QAHM_NAME . ' = ' . QAHM_NAME . ' || {}; let ' . QAHM_NAME . 'Obj = ' . wp_json_encode( $scripts ) . '; ' . QAHM_NAME . ' = Object.assign( ' . QAHM_NAME . ', ' . QAHM_NAME . 'Obj );', 'before' );

		// localize
		$localize = $this->get_common_localize_script();
		$localize['powerup_title'] = esc_html__( 'Upgraded', 'qa-heatmap-analytics' );
		$localize['powerup_text']  = sprintf( esc_html__( 'Your license authentication is done successfully. %s Enjoy the advanced QA!', 'qa-heatmap-analytics' ), '<br>' );
		wp_localize_script( QAHM_NAME . '-common', QAHM_NAME . 'l10n', $localize );
	}

	/**
	 * ページの表示
	 */
	public function create_html() {
		if( ! $this->is_enqueue_jquery() ) {
			$this->view_not_enqueue_jquery_html();
			return;
		}
		
		if( ! $this->is_email_regist() ) {
			$this->view_noticemail_html();
			return;
		}

		if( $this->is_maintenance() ) {
			$this->view_maintenance_html();
			return;
		}

		global $qahm_time;
		$plan      = $this->get_plan();
		if( $this->wrap_get_option( 'license_plan' ) === 'friend' ) {
			$is_friend_plan = true;
		} else {
			$is_friend_plan = false;
		}
		$key       = $this->wrap_get_option( 'license_key' );
		$id        = $this->wrap_get_option( 'license_id' );
		$act_time  = $this->wrap_get_option( 'license_activate_time' );
		$act_time  = $qahm_time->unixtime_to_str( $act_time );
		$input_read = '';
		if ( $plan || $is_friend_plan ) {
			$input_read = ' readonly';
		}
		?>

		<div id="<?php esc_attr_e( basename( __FILE__, '.php' ) ); ?>" class="qahm-admin-page">
			<div class="wrap">
				<h1>QA <?php esc_html_e( 'License Activation', 'qa-heatmap-analytics' ); ?></h1>
				<?php $this->view_announce_html(); ?>
				<?php 
				$license_plan = $this->get_plan();
				if ( ! $license_plan ) {
				$msg_yuryouikaga = '<div id="bl_pay_announce" class="qahm_using_free_announce"><span class="qahm_margin-right4"><i class="fas fa-exclamation-triangle"></i></span><span class="qahm_fontsize12em">';
                    $msg_yuryouikaga .=	sprintf( esc_html__( 'You are using the FREE version of QA. Get full-fledged and enrich analysis with %1$s QA Upgrade Options %2$s.', 'qa-heatmap-analytics' ), '<a href="https://quarka.org/dataplan-upgrade/" target="_blank" rel="noopener">', '</a>' ); //'現在『無料版』をご利用中です。QA有料機能を使えば、より本格的で充実した解析ができます。'
                    $msg_yuryouikaga .= '</span></div>';
                    echo $msg_yuryouikaga;
				}
				?>

				<form method="post" action="">
					<?php wp_nonce_field( self::NONCE_ACTION, self::NONCE_NAME ); ?>

					<table class="form-table">
						<tbody>
							<tr>
								<th scope="row">
									<label for="license_key">
										<?php esc_attr_e( 'License key', 'qa-heatmap-analytics' ); ?>
									</label>
								</th>
								<td>
									<input name="license_key" type="text" id="license_key" value="<?php echo esc_attr( $key ); ?>" class="regular-text"<?php echo $input_read; ?>>
									<?php if ( ! $plan && ! $is_friend_plan ) { ?>
										<button type="button" onclick="window.open( 'https://quarka.org/plan/', '_blank' ); return false;" class="button button-primary">
											<?php esc_html_e( 'Go to get license', 'qa-heatmap-analytics' ); ?>
										</button>
									<?php } ?>
								</td>
							</tr>

							<tr>
								<th scope="row">
									<label for="license_id">
										<?php esc_html_e( 'Your email', 'qa-heatmap-analytics' ); ?>
									</label>
								</th>
								<td>
									<input name="license_id" type="email" id="license_id" value="<?php echo esc_attr( $id ); ?>" class="regular-text" placeholder="test@example.com"<?php echo $input_read; ?>>
								</td>
							</tr>

							<tr>
								<td colspan="2">
									<?php if ( ! $plan && ! $is_friend_plan ) { ?>
										<p class="submit">
											<input type="hidden" name="license_cmd" value="check">
											<input type="hidden" name="license_url" value="https://mem.quarka.org/tsushin/">
											<input type="submit" name="submit" id="submit" class="button button-primary" value="<?php esc_attr_e( 'Activate the license', 'qa-heatmap-analytics' ); ?>">
										</p>
									<?php } else { ?>
										<p class="submit">
											<input type="hidden" name="license_cmd" value="deactivate">
											<input type="hidden" name="license_url" value="https://mem.quarka.org/tsushin/">
											<input type="submit" name="submit" id="submit" class="button button-primary" value="<?php esc_attr_e( 'Deactivate this license', 'qa-heatmap-analytics' ); ?>">
										</p>
									<?php } ?>
								</td>
							</tr>
							
							<?php if ( $plan || $is_friend_plan ) { ?>
								<tr>
									<th scope="row">
										<?php esc_html_e( 'Last authentication time', 'qa-heatmap-analytics' ); ?>
									</th>
									<td>
										<?php echo esc_html( $act_time ); ?>
									</td>
								</tr>

								<tr>
									<th scope="row">
										<?php esc_html_e( 'Connection to Google Data Studio', 'qa-heatmap-analytics' ); ?>
									</th>
									<td>
										<?php
											if ( $this->check_plan( 'api_google_data_studio' ) ) {
												echo 'on';
											} else {
												echo 'off';
											}
										?>
									</td>
								</tr>
								<tr>
									<th scope="row">
										<?php esc_html_e( 'Retain PV data 5 yrs', 'qa-heatmap-analytics' ); ?>
									</th>
									<td>
										<?php
											if ( $this->check_plan( 'save_pv' ) ) {
												echo 'on';
											} else {
												echo 'off';
											}
										?>
									</td>
								</tr>
								<tr>
									<th scope="row">
										<?php esc_html_e( 'Retain Event data 5 yrs', 'qa-heatmap-analytics' ); ?>
									</th>
									<td>
										<?php
											if ( $this->check_plan( 'save_hm' ) ) {
												echo 'on';
											} else {
												echo 'off';
											}
										?>
									</td>
								</tr>
								<tr>
									<th scope="row">
										<?php esc_html_e( 'Collect Event on All Pages', 'qa-heatmap-analytics' ); ?>
									</th>
									<td>
										<?php
											if ( $this->check_plan( 'all_hm' ) ) {
												echo 'on';
											} else {
												echo 'off';
											}
										?>
									</td>
								</tr>
								<tr>
									<th scope="row">
										<?php esc_html_e( 'Collect Event on max Three Pages', 'qa-heatmap-analytics' ); ?>
									</th>
									<td>
										<?php
											if ( $this->check_plan( 'three_hm' ) ) {
												echo 'on';
											} else {
												echo 'off';
											}
										?>
									</td>
								</tr>
								<tr>
									<th scope="row">
										<?php esc_html_e( 'Increaced PV Limit', 'qa-heatmap-analytics' ); ?>
									</th>
									<td>
										<?php
											if ( $this->check_plan( 'support_10-30_pv' ) ) {
												echo 'on';
											} else {
												echo 'off';
											}
										?>
									</td>
								</tr>
								<tr>
									<th scope="row">
										<?php esc_html_e( '3 Goals setting', 'qa-heatmap-analytics' ); ?>
									</th>
									<td>
										<?php
											if ( $this->check_plan( 'goal_3' ) ) {
												echo 'on';
											} else {
												echo 'off';
											}
										?>
									</td>
								</tr>
								<tr>
									<th scope="row">
										<?php esc_html_e( '10 Goals setting', 'qa-heatmap-analytics' ); ?>
									</th>
									<td>
										<?php
											if ( $this->check_plan( 'goal_10' ) ) {
												echo 'on';
											} else {
												echo 'off';
											}
										?>
									</td>
								</tr>
							<?php } ?>
						</tbody>
					</table>
				</form>
			</div>
		</div>
		<?php

		// ライセンスページで1度のみ表示するメッセージであればこの段階で非表示設定にする
		$msg_ary = $this->wrap_get_option( 'license_message' );
		if ( $msg_ary ) {
			foreach ( $msg_ary as &$msg ) {
				if ( $msg['view'] === QAHM_License::MESSAGE_VIEW['license'] ) {
					$msg['view'] = QAHM_License::MESSAGE_VIEW['hidden'];
				}
			}
			$this->wrap_update_option( 'license_message', $msg_ary );
		}
	}

	/**
	 * 設定画面の項目をデータベースに保存する
	 */
	public function save_config() {
		if ( empty( $wp_filesystem ) ) {
			require_once ABSPATH . 'wp-admin/includes/file.php';
			$creds = false;

			$access_type = get_filesystem_method();
			if ( $access_type === 'ftpext' ) {
				$creds = request_filesystem_credentials( '', '', false, false, null );
			}
			if ( ! WP_Filesystem( $creds ) ) {
				throw new Exception( 'WP_Filesystem Invalid Credentials' );
			}
		}

		$post = array();
		// nonceで設定したcredentialのチェック
		// 設定画面
		if ( isset( $_POST[ self::NONCE_NAME ] ) && $_POST[ self::NONCE_NAME ] ) {
			if ( check_admin_referer( self::NONCE_ACTION, self::NONCE_NAME ) ) {
				global $qahm_license;
				
				$cmd = $this->wrap_filter_input( INPUT_POST, 'license_cmd' );
				$key = $this->wrap_filter_input( INPUT_POST, 'license_key' );
				$id  = $this->wrap_filter_input( INPUT_POST, 'license_id' );
				$url = $this->wrap_filter_input( INPUT_POST, 'license_url' );

				// フォームの値を更新
				$this->wrap_update_option( 'license_key', $key );
				$this->wrap_update_option( 'license_id', $id );

				if ( 'check' === $cmd ) {
					// アクティベート
					$res = $qahm_license->activate( $key, $id, QAHM_License::MESSAGE_VIEW['license'], $url );

				} elseif ( 'deactivate' === $cmd ) {
					// ディアクティベート
					$res = $qahm_license->deactivate( $key, $id,QAHM_License::MESSAGE_VIEW['license'], $url );
				}
			}
		}
	}

	/**
	* ライセンスメッセージのクリア
	*/
	public function ajax_clear_license_message() {
		$no  = (int) $this->wrap_filter_input( INPUT_POST, 'no' );

		$msg_ary  = $this->wrap_get_option( 'license_message' );
		if ( ! $msg_ary ) {
			die();
		}

		foreach ( $msg_ary as &$msg ) {
			if( $msg['no'] === $no ) {
				$msg['view'] = QAHM_License::MESSAGE_VIEW['hidden'];
			}
		}
		
		$this->wrap_update_option( 'license_message', $msg_ary );
		die();
	}
} // end of class
