<?php
/**
 * ヒートマップビューで様々な操作をやりやすくするクラス（予定）
 *
 * @package qa_heatmap
 */

$qahm_view_heatmap = new QAHM_View_Heatmap();

class QAHM_View_Heatmap extends QAHM_View_base {

	const ATTENTION_LIMIT_TIME = 4;

	public function __construct() {
		$this->regist_ajax_func( 'ajax_create_heatmap_file' );
		$this->regist_ajax_func( 'ajax_init_heatmap_view' );
		//$this->regist_ajax_func( 'ajax_change_rec_checkbox' );
		//$this->regist_ajax_func( 'ajax_change_heatmap_view_bar' );
		
		add_action( 'init', array( $this, 'init_wp_filesystem' ) );
	}

	public function get_heatmap_view_work_dir_url() {
		return parent::get_data_dir_url() . 'heatmap-view-work/';
	}
	
	/**
	 * ヒートマップ表示用のファイルを作成
	 */
	public function ajax_create_heatmap_file() {
		global $qahm_log;

		try {
			$version_id = (int) $this->wrap_filter_input( INPUT_POST, 'version_id' );
			
			$this->write_wp_load_path();
			$this->create_heatmap_file( $version_id );

			$heatmap_view_url  = esc_url( plugin_dir_url( __FILE__ ) . 'heatmap-view.php' ) . '?';
			$heatmap_view_url .= 'version_id=' . $version_id;

			echo wp_json_encode( $heatmap_view_url );

		} catch ( Exception $e ) {
			$log = $qahm_log->error( $e->getMessage() );
			echo wp_json_encode( $log );

		} finally {
			die();
		}
	}

	/**
	 * ヒートマップビュー用の一時ファイルを作成
	 */
	public function create_heatmap_file( $version_id ) {
		global $qahm_db;
		global $wp_filesystem;
		global $qahm_log;
		
		$qa_pv_log   = json_decode( $this->wrap_filter_input( INPUT_POST, 'view_pv_ary' ), true );
		$start_date  = $this->wrap_filter_input( INPUT_POST, 'start_date' );
		$end_date    = $this->wrap_filter_input( INPUT_POST, 'end_date' );
		$base_html   = null;
		$heatmap_view_work_dir = $this->get_data_dir_path( 'heatmap-view-work' );
		$this->wrap_mkdir( $heatmap_view_work_dir );

		// base.htmlをdbから取得
		$base_html            = null;
		$base_selector        = null;
		$qa_pages             = null;
		$qa_page_version_hist = null;
		$page_id              = null;
		$base_url             = null;
		$version_no           = null;
		$device_name          = null;
		$device_id            = null;
		$wp_qa_type           = null;
		$wp_qa_id             = null;

		$merge_att_scr_ary_v1 = array();
		$merge_att_scr_ary_v2 = array();
		$merge_click_ary   = array();
		$data_num          = 0;
		$time_on_page      = 0;

		// 以下ネストが深くならないよう処理を区切っている
		$table_name            = $qahm_db->prefix . 'view_page_version_hist';
		$query                 = "SELECT * FROM " . $table_name . " WHERE version_id = %d";
		$query                 = $qahm_db->prepare( $query, $version_id );
		$qa_page_version_hist  = $qahm_db->get_results( $query, ARRAY_A );
		
		if ( $qa_page_version_hist ) {
			$hist          = $qa_page_version_hist[0];
			$version_id    = (int) $hist['version_id'];
			$page_id       = (int) $hist['page_id'];
			$device_id     = (int) $hist['device_id'];
			$device_name   = $this->device_id_to_device_name( $device_id );
			$version_no    = (int) $hist['version_no'];
			$base_html     = $hist['base_html'];
			$base_selector = $hist['base_selector'];

			$table_name = 'qa_pages';
			$query      = 'SELECT wp_qa_type,wp_qa_id,url FROM ' . $qahm_db->prefix . $table_name . ' WHERE page_id = %d';
			$qa_pages   = $qahm_db->get_results( $qahm_db->prepare( $query, $page_id ), ARRAY_A );

			// 引数でview_pv配列を渡されないとき（ヒートマップ管理画面からアクセスするとき）は、この時点でview_pvを検索
			if ( ! $qa_pv_log ) {
				$table_name = 'view_pv';
				$query     = 'SELECT pv_id,is_raw_p,is_raw_c,is_raw_e FROM ' . $qahm_db->prefix . $table_name . ' WHERE version_id = %d';
				$qa_pv_log  = $qahm_db->get_results( $qahm_db->prepare( $query, $version_id ), ARRAY_A );

				// view_pv配列が存在しないときはDBからデータ取得
				if ( ! $qa_pv_log ) {
					if ( $start_date && $end_date ) {
						$query     = "select pv_id,raw_p,raw_c,raw_e from " . $qahm_db->prefix . "qa_pv_log where version_id = %d AND access_time BETWEEN %s AND %s";
						$qa_pv_log = $qahm_db->get_results( $qahm_db->prepare( $query, $version_id, $start_date, $end_date ), ARRAY_A );
					} else {
						$query     = "select pv_id,raw_p,raw_c,raw_e from " . $qahm_db->prefix . "qa_pv_log where version_id = %d";
						$qa_pv_log = $qahm_db->get_results( $qahm_db->prepare( $query, $version_id ), ARRAY_A );
					}
				}
			}
		}

		if ( $qa_pages ) {
			$page       = $qa_pages[0];
			$wp_qa_type = $page['wp_qa_type'];
			$wp_qa_id   = $page['wp_qa_id'];
			$base_url   = $page['url'];
		}

		if ( $qa_pv_log ) {
			// 100分率 + 精読率100%の分配列を用意
			for ( $iii = 0; $iii < 100 + 1; $iii++ ) {
				$merge_att_scr_ary_v1[ $iii ] = array( $iii, 0, 0, 0 );
			}

			// base_selectorの配列
			$click_idx         = 0;
			$base_selector_ary = explode( "\t", $base_selector );

			$total_stay_time = 0;
			
			$view_pv_dir   = $this->get_data_dir_path( 'view' ) . $this->get_tracking_id() . '/view_pv/';
			$raw_p_dirlist = $this->wrap_dirlist( $view_pv_dir . 'raw_p/' );
			$raw_c_dirlist = $this->wrap_dirlist( $view_pv_dir . 'raw_c/' );

			$raw_p_dir_idx = 0;
			$raw_p_data_ary = null;
			$raw_p_data_pv_id_min = 0;
			$raw_p_data_pv_id_max = 0;
			$raw_c_dir_idx = 0;
			$raw_c_data_ary = null;
			$raw_c_data_pv_id_min = 0;
			$raw_c_data_pv_id_max = 0;

			foreach ( $qa_pv_log as $pv_log ) {
				$raw_p_tsv = null;
				$raw_c_tsv = null;

				if ( array_key_exists( 'is_raw_p', $pv_log ) ) {
					// view_pv
					if ( $pv_log['is_raw_p'] || $pv_log['is_raw_c'] || $pv_log['is_raw_e'] ) {
						$pv_id = $pv_log['pv_id'];

						if ( $pv_log['is_raw_p'] ) {
							$raw_p_dir_first_search = false;

							if ( $raw_p_data_ary && $raw_p_data_pv_id_min <= $pv_id && $raw_p_data_pv_id_max >= $pv_id ) {
								foreach ( $raw_p_data_ary as $raw_p_data ) {
									if( $pv_id !== $raw_p_data['pv_id'] ) {
										continue;
									}
									$raw_p_tsv = $raw_p_data['raw_p'];
									break;
								}
							}

							if ( ! $raw_p_tsv ) {
								for ( $i = $raw_p_dir_idx, $raw_p_file_max = count( $raw_p_dirlist ); $i < $raw_p_file_max; $i++ ) {
									preg_match( '/_(\d+)-(\d+)_/', $raw_p_dirlist[$i]['name'], $matches );
									if ( ! array_key_exists( 1, $matches ) || ! array_key_exists( 2, $matches ) ) {
										continue;
									}

									if ( $matches[1] > $pv_id || $matches[2] < $pv_id ) {
										continue;
									}
		
									$raw_p_data_ary = $this->wrap_unserialize( $this->wrap_get_contents( $view_pv_dir . 'raw_p/' . $raw_p_dirlist[$i]['name'] ) );
									if ( ! $raw_p_dir_first_search ) {
										$raw_p_dir_first_search = true;
										$raw_p_dir_idx = $i;
										$raw_p_data_pv_id_min = $matches[1];
										$raw_p_data_pv_id_max = $matches[2];
									}

									foreach ( $raw_p_data_ary as $raw_p_data ) {
										if( $pv_id !== $raw_p_data['pv_id'] ) {
											continue;
										}
										$raw_p_tsv = $raw_p_data['raw_p'];
										break;
									}
		
									if( $raw_p_tsv ) {
										break;
									}
								}
							}
						}
	
						if ( $pv_log['is_raw_c'] ) { 
							$raw_c_dir_first_search = false;

							if ( $raw_c_data_ary && $raw_c_data_pv_id_min <= $pv_id && $raw_c_data_pv_id_max >= $pv_id ) {
								foreach ( $raw_c_data_ary as $raw_c_data ) {
									if( $pv_id !== $raw_c_data['pv_id'] ) {
										continue;
									}
									$raw_c_tsv = $raw_c_data['raw_c'];
									break;
								}
							}

							if ( ! $raw_c_tsv ) {
								for ( $i = $raw_c_dir_idx, $raw_c_file_max = count( $raw_c_dirlist ); $i < $raw_c_file_max; $i++ ) {
									preg_match( '/_(\d+)-(\d+)_/', $raw_c_dirlist[$i]['name'], $matches );
									if ( ! array_key_exists( 1, $matches ) || ! array_key_exists( 2, $matches ) ) {
										continue;
									}

									if ( $matches[1] > $pv_id || $matches[2] < $pv_id ) {
										continue;
									}

									$raw_c_data_ary = $this->wrap_unserialize( $this->wrap_get_contents( $view_pv_dir . 'raw_c/' . $raw_c_dirlist[$i]['name'] ) );
									if ( ! $raw_c_dir_first_search ) {
										$raw_c_dir_first_search = true;
										$raw_c_dir_idx = $i;
										$raw_c_data_pv_id_min = $matches[1];
										$raw_c_data_pv_id_max = $matches[2];
									}

									foreach ( $raw_c_data_ary as $raw_c_data ) {
										if( $pv_id !== $raw_c_data['pv_id'] ) {
											continue;
										}
										$raw_c_tsv = $raw_c_data['raw_c'];
										break;
									}

									if( $raw_c_tsv ) {
										break;
									}
								}
							}
						}
	
						//$query     = "select raw_p,raw_c from " . $wpdb->prefix . "qa_pv_log where pv_id=%d";
						//$qa_pv_log = $wpdb->get_results( $wpdb->prepare( $query, $view_pv['pv_id'] ) );
						//$pv_log    = $qa_pv_log[0];
						
						if ( $raw_p_tsv || $raw_c_tsv ) {
							$data_num++;
						}
					}
				} else {
					// qa_pv_log
					if ( $pv_log['raw_p'] || $pv_log['raw_c'] || $pv_log['raw_e'] ) {
						$raw_p_tsv = $pv_log['raw_p'];
						$raw_c_tsv = $pv_log['raw_c'];

						$data_num++;
					}
				}

				if ( $raw_p_tsv ) {

					$raw_p_ary   = $this->convert_tsv_to_array( $raw_p_tsv );
					$exit_idx    = -1;

					// 滞在時間
					$ver = (int) $raw_p_ary[self::DATA_COLUMN_HEADER][self::DATA_HEADER_VERSION];
					if ( $ver === 2 ) {
						for ( $raw_p_idx = self::DATA_COLUMN_BODY, $raw_p_max = count($raw_p_ary); $raw_p_idx < $raw_p_max; $raw_p_idx++ ) {
							$p = $raw_p_ary[$raw_p_idx];
							if ( $p[ self::DATA_POS_2['STAY_HEIGHT'] ] === 'a' ) {
								// あとでこの処理を詰める
								break;
							}

							$stay_time = min( (int) $p[ self::DATA_POS_2['STAY_TIME'] ], self::ATTENTION_LIMIT_TIME );
							$merge_att_scr_idx = count( $merge_att_scr_ary_v2 );
							$is_find = false;
							if ( $merge_att_scr_idx > 0 ) {
								// 既に高さのデータがあれば滞在時間を加算
								foreach ( $merge_att_scr_ary_v2 as $val_idx => $val_ary ) {
									if ( $val_ary[self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_HEIGHT']] === (int) $p[ self::DATA_POS_2['STAY_HEIGHT'] ] ) {
										$merge_att_scr_idx = $val_idx;
										$merge_att_scr_ary_v2[ $merge_att_scr_idx ][ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_TIME'] ] += $stay_time;
										$merge_att_scr_ary_v2[ $merge_att_scr_idx ][ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_NUM'] ]++;
										$is_find = true;
										break;
									}
								}
							}

							if ( ! $is_find ) {
								$merge_att_scr_ary_v2[ $merge_att_scr_idx ] = array(
									(int) $p[ self::DATA_POS_2['STAY_HEIGHT'] ],
									$stay_time,
									1,
									0
								);
							}

							// 離脱位置を更新。既にソートされた配列なので比較の必要なし
							$exit_idx = $merge_att_scr_idx;

							// 合計滞在時間に加算
							$total_stay_time += $stay_time;
						}

						// body部が存在しなかったユーザーの対策。離脱位置を強制的に0の部分にする。
						// これにより、ヒートマップビューのデータ数とスクロールマップトップのデータ数との見た目上の整合性を合わせる
						if ( $exit_idx === -1 ) {
							if ( count( $merge_att_scr_ary_v2 ) > 0 ) {
								foreach ( $merge_att_scr_ary_v2 as $val_idx => $val_ary ) {
									if ( $val_ary[self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_HEIGHT']] === 0 ) {
										$exit_idx = $val_idx;
										break;
									}
								}
							}
							if ( $exit_idx === -1 ) {
								$exit_idx = count( $merge_att_scr_ary_v2 );
								$merge_att_scr_ary_v2[ $exit_idx ] = array(
									0,
									0,
									1,
									0
								);
							}
						}
						
						// 離脱ユーザーの位置
						$merge_att_scr_ary_v2[ $exit_idx ][ self::DATA_MERGE_ATTENTION_SCROLL_2['EXIT_NUM'] ]++;

					} elseif ( $ver === 1 ) {
						for ( $raw_p_idx = self::DATA_COLUMN_BODY, $raw_p_max = count($raw_p_ary); $raw_p_idx < $raw_p_max; $raw_p_idx++ ) {
							$p = $raw_p_ary[$raw_p_idx];
							if ( $p[ self::DATA_POS_1['PERCENT_HEIGHT'] ] === 'a' ) {
								$exit_idx = 100;
							} else {
								// 滞在位置
								$att_scr_idx = (int) $p[ self::DATA_POS_1['PERCENT_HEIGHT'] ];
								// 滞在位置が0地点なら1に補正する
								if ( $att_scr_idx === 0 ) {
									$att_scr_idx = 1;
								}

								// 滞在地点ごとの合計滞在時間を求める
								$stay_time = min( (int) $p[ self::DATA_POS_1['TIME_ON_HEIGHT'] ], self::ATTENTION_LIMIT_TIME );
								$merge_att_scr_ary_v1[ $att_scr_idx ][ self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_TIME'] ] += $stay_time;
								$merge_att_scr_ary_v1[ $att_scr_idx ][ self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_NUM'] ]++;

								// 離脱位置を更新。既にソートされた配列なので比較の必要なし
								$exit_idx = $att_scr_idx;

								// 合計滞在時間に加算
								$total_stay_time += $stay_time;
							}
						}

						// body部が存在しなかったユーザーの対策。離脱位置を強制的に1の部分にする。
						// これにより、ヒートマップビューのデータ数とスクロールマップトップのデータ数との見た目上の整合性を合わせる
						if ( $exit_idx === -1 ) {
							$merge_att_scr_ary_v1[ 1 ][ self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_NUM'] ]++;
							$exit_idx = 1;
						}
	
						// 離脱ユーザーの位置
						$merge_att_scr_ary_v1[ $exit_idx ][ self::DATA_MERGE_ATTENTION_SCROLL_1['EXIT_NUM'] ]++;
					}
				}

				if ( $raw_c_tsv && $base_selector ) {
					$raw_c_ary = $this->convert_tsv_to_array( $raw_c_tsv );

					foreach ( $raw_c_ary as $index => $c ) {
						if ( $index === self::DATA_COLUMN_HEADER ) {
							// header部。現在は何もしない
						} else {
							// body部
							$merge_click_ary[ $click_idx ] = array();
							$merge_click_ary[ $click_idx ][ self::DATA_MERGE_CLICK_1['SELECTOR_NAME'] ] = $base_selector_ary[ $c[ self::DATA_CLICK_1['SELECTOR_NAME'] ] ];
							$merge_click_ary[ $click_idx ][ self::DATA_MERGE_CLICK_1['SELECTOR_X'] ]    = $c[ self::DATA_CLICK_1['SELECTOR_X'] ];
							$merge_click_ary[ $click_idx ][ self::DATA_MERGE_CLICK_1['SELECTOR_Y'] ]    = $c[ self::DATA_CLICK_1['SELECTOR_Y'] ];
							$click_idx++;
						}
					}
				}
			}

			if ( $data_num > 0 ) {
				$time_on_page = round( $total_stay_time / $data_num, 2 );

				$exit_num = 0;
				foreach( $merge_att_scr_ary_v2 as $val_ary ) {
					$exit_num += $val_ary[ self::DATA_MERGE_ATTENTION_SCROLL_2['EXIT_NUM'] ];
				}
				foreach( $merge_att_scr_ary_v1 as $val_ary ) {
					$exit_num += $val_ary[ self::DATA_MERGE_ATTENTION_SCROLL_1['EXIT_NUM'] ];
				}

				// raw_pが存在しない、かつraw_eが存在するデータがある（昔のデータやファイル作成失敗？）
				// その対策に、データ数を補正する。離脱位置は1の部分
				if ( $data_num !== $exit_num ) {
					$dif_num = $data_num - $exit_num;

					// 離脱ユーザーの位置
					$merge_att_scr_ary_v1[ 1 ][ self::DATA_MERGE_ATTENTION_SCROLL_1['EXIT_NUM'] ] += $dif_num;
					$merge_att_scr_ary_v1[ 1 ][ self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_NUM'] ] += $dif_num;
				}
			}

			// 合算したデータの平均値を求める
			$merge_max = count( $merge_att_scr_ary_v2 );
			if ( $merge_max > 0 ) {
				for ( $merge_idx = 0; $merge_idx < $merge_max; $merge_idx++ ) {
					if ( $merge_att_scr_ary_v2[$merge_idx][ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_NUM'] ] > 1 ) {
						$merge_att_scr_ary_v2[$merge_idx][ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_TIME'] ] /= $merge_att_scr_ary_v2[$merge_idx][ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_NUM'] ];
						$merge_att_scr_ary_v2[$merge_idx][ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_TIME'] ]  = round( $merge_att_scr_ary_v2[$merge_idx][ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_TIME'] ], 3 );
					}
				}
			}

			$merge_max = count( $merge_att_scr_ary_v1 );
			if ( $merge_max > 0 ) {
				for ( $merge_idx = 0; $merge_idx < $merge_max; $merge_idx++ ) {
					if ( $merge_att_scr_ary_v1[$merge_idx][ self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_NUM'] ] > 1 ) {
						$merge_att_scr_ary_v1[$merge_idx][ self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_TIME'] ] /= $merge_att_scr_ary_v1[$merge_idx][ self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_NUM'] ];
						$merge_att_scr_ary_v1[$merge_idx][ self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_TIME'] ]  = round( $merge_att_scr_ary_v1[$merge_idx][ self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_TIME'] ], 3 );
					}
				}
			}
		}

		// 配列のソート
		if ( $merge_att_scr_ary_v2 ) {
			// ソート後、tsvに変換して保存
			$sort_ary = array();
			foreach ( $merge_att_scr_ary_v2 as $val_idx => $val_ary ) {
				$sort_ary[$val_idx] = $val_ary[ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_HEIGHT'] ];
			}
			array_multisort( $sort_ary, SORT_ASC, $merge_att_scr_ary_v2 );
		}

		// dbにbase_htmlが存在しない場合は作る
		if ( ! $base_html ) {
			$http_response_header = null;
			$options              = $this->get_stream_options( $device_name );
			$base_html            = file_get_contents( $base_url, false, stream_context_create( $options ) );
			if ( $this->is_zip( $base_html ) ) {
				$temphtml = gzdecode( $base_html );
				if ( $temphtml !== false ) {
					$base_html = $temphtml;
				}
			}

			// 「$http_response_header[0]」にはステータスコードがセットされている
			if ( $http_response_header ) {
				preg_match( '/HTTP\/1\.[0|1|x] ([0-9]{3})/', $http_response_header[0], $matches );
				$status_code = $matches[1];

				switch ( $status_code ) {
					case 200:
						/*
						DBから取ってこない可能性があるのでこのコードは封印
						if ( $version_id ) {
							// base_htmlの更新
							$wpdb->update(
								$wpdb->prefix . 'qa_page_version_hist',
								array(
									'base_html' => $base_html,
								),
								array(
									'version_id' => $version_id,
								),
								array(
									'%s',
								),
								array(
									'%d',
								)
							);
						}
						*/
						break;
					case 404:
						throw new Exception( 'As a result of doing file_get_contents, 404 was returned' );
						break;
					case 500:
						throw new Exception( 'As a result of doing file_get_contents, 500 was returned' );
						break;
					default:
						throw new Exception( 'As a result of doing file_get_contents, some kind of error was returned' );
						break;
				}
				// $http_response_headerに値が無ければ404として扱う
			} else {
				throw new Exception( 'As a result of doing file_get_contents, 404 was returned' );
			}
		}

		// baseが存在した場合、cap.phpを作成する
		if ( $base_html ) {
			// capはbaseを加工
			$cap_path    = $heatmap_view_work_dir . $version_id . '-cap.php';
			//$cap_content = $this->opt_html( $cap_path, $base_html, $type, $id, $ver, $device_name );
			$cap_content = $this->opt_base_html( $cap_path, $base_html, $base_url, $device_name );

			if ( $cap_content ) {
				// cap
				if ( ! $wp_filesystem->put_contents( $cap_path, $cap_content ) ) {
					$qahm_log->error( '$wp_filesystem->put_contents()に失敗しました。パス：' . $cap_path );
				}

				// マージファイル
				if ( $merge_att_scr_ary_v2 ) {
					$head = array(
						array(
							self::DATA_HEADER_VERSION => 2,
						)
					);
					$merge_att_scr_ary_v2 = array_merge( $head, $merge_att_scr_ary_v2 );
					$merge_att_scr_tsv = $this->convert_array_to_tsv( $merge_att_scr_ary_v2 );

					$path = $heatmap_view_work_dir . $version_id . '-merge-as-v2.php';
					$this->wrap_put_contents( $path, $merge_att_scr_tsv );
				}

				if ( $merge_att_scr_ary_v1 ) {
					$head = array(
						array(
							self::DATA_HEADER_VERSION => 1,
						)
					);
					$merge_att_scr_ary_v1 = array_merge( $head, $merge_att_scr_ary_v1 );
					$merge_att_scr_tsv = $this->convert_array_to_tsv( $merge_att_scr_ary_v1 );

					$path = $heatmap_view_work_dir . $version_id . '-merge-as-v1.php';
					$this->wrap_put_contents( $path, $merge_att_scr_tsv );
				}

				if ( $merge_click_ary ) {
					$head = array(
						array(
							self::DATA_HEADER_VERSION => 1,
						)
					);
					$merge_click_ary = array_merge( $head, $merge_click_ary );
					$merge_click_tsv = $this->convert_array_to_tsv( $merge_click_ary );

					$path = $heatmap_view_work_dir . $version_id . '-merge-c.php';
					$this->wrap_put_contents( $path, $merge_click_tsv );
				}

				// 情報を格納するinfoファイル
				// iniファイルと同じような書き方。シンプルにしたいが為にセクションは無し
				$info_str  = '';
				$info_str .= 'base_url=' . $base_url . PHP_EOL;
				$info_str .= 'data_num=' . $data_num . PHP_EOL;
				$info_str .= 'wp_qa_type=' . $wp_qa_type . PHP_EOL;
				$info_str .= 'wp_qa_id=' . $wp_qa_id . PHP_EOL;
				$info_str .= 'version_no=' . $version_no . PHP_EOL;
				$info_str .= 'device_name=' . $device_name . PHP_EOL;
				$info_str .= 'time_on_page=' . $time_on_page;

				$path = $heatmap_view_work_dir . $version_id . '-info.php';
				$this->wrap_put_contents( $path, $info_str );
			}
		}
	}

	public function get_html_bar_text( $id, $html, $tooltip, $both = false, $link = '' ) {
		$clear_both = '';
		if ( $both ) {
			$clear_both = ' style="clear: both;"'; 
		}
		$link_start = '';
		$link_end   = '';
		if ( $link ) {
			$link_start = '<a href="' . $link . '" target="_blank" rel="noopener noreferrer"">';
			$link_end   = '</a>';
		}

		return <<< EOT
			<li id="{$id}"{$clear_both}>
				{$link_start}
				<span class="qahm-tooltip-bottom" data-qahm-tooltip="{$tooltip}">
					{$html}
				</span>
				{$link_end}
			</li>
EOT;
	}

	public function get_html_bar_checkbox( $id, $html, $tooltip, $is_check, $is_mobile ) {
		$check_html = '';
		if ( $is_check ) {
			$check_html = ' checked';
		}

		$mobile_html = '';
		if ( $is_mobile ) {
			$mobile_html = '-mobile';
		}

		return <<< EOT
			<li id="{$id}-checkbox{$mobile_html}">
				<label class="control control--checkbox">
					<span class="qahm-tooltip-bottom" data-qahm-tooltip="{$tooltip}">
						{$html}
						<input class="heatmap-bar-check {$id}" type="checkbox"{$check_html} disabled>
						<div class="control__indicator"></div>
					</span>
				</label>
			</li>
EOT;
	}

	// ヒートマップ表示画面上で必要な初期情報を取得
	public function ajax_init_heatmap_view() {
		$data = array();

		global $wp_filesystem;
		global $qahm_recterm;
		$recterm = $qahm_recterm->get_all_record();

		$type       = $this->wrap_filter_input( INPUT_POST, 'type' );
		$id         = $this->wrap_filter_input( INPUT_POST, 'id' );
		$ver        = $this->wrap_filter_input( INPUT_POST, 'ver' );
		$dev        = $this->wrap_filter_input( INPUT_POST, 'dev' );
		$version_id = $this->wrap_filter_input( INPUT_POST, 'version_id' );

		// cap.phpは一日ごとの更新のため、リアルタイムに変わってほしい変数や
		// QAHMバーを初期化する際に必須の情報を受け取る
		$data['const_debug_level'] = QAHM_DEBUG_LEVEL;
		$data['const_debug']       = QAHM_DEBUG;
		$data['locale']            = get_locale();
		$data['data_num']          = 0;            // データ数はcap.phpに移動予定
		$data['ver_max']           = 1;				// 後々修正 imai
		$data['heatmap']           = false;
		$data['attention']         = false;
		$data['rec_flag']          = (int) $qahm_recterm->get_field( 'rec_flag', $type, $id );
		$data['is_raw_save_all']   = $this->wrap_get_option( 'is_raw_save_all' );
		$data['free_rec_flag']     = false;
		//if ( ! $this->wrap_get_option( 'license_auth' ) ) {
			$rec_page = 0;
			for ( $i = 0, $rec_cnt = count( $recterm ); $i < $rec_cnt; $i++ ) {
				if ( $recterm[ $i ]['rec_flag'] ) {
					$rec_page++;
					if ( $rec_page <= 1 &&
						$recterm[ $i ]['type'] == $type &&
						$recterm[ $i ]['ids'] == $id ) {
						// 何もしない
					} else {
						$data['free_rec_flag'] = true;
						break;
					}
				}
			}
		//}

		$heatmap_view_work_dir = $this->get_data_dir_path( 'heatmap-view-work' );

		$data['merge_c']  = null;
		$data['merge_as'] = null;
		$data['data_num'] = 0;

		// infoファイルの読み込み
		$info_ary = $wp_filesystem->get_contents_array( $heatmap_view_work_dir . $version_id . '-info.php' );
		foreach ( $info_ary as $info ) {
			$info_param = explode( '=', $info );
			if ( count( $info_param ) === 2 ) {
				switch ( $info_param[0] ) {
					case 'data_num':
						$data['data_num'] = (int) $info_param[1];
						break;
				}
			}
		}


		$lists = $this->wrap_dirlist( $heatmap_view_work_dir );
		foreach ( $lists as $list ) {			
			// 内部でデータヘッダーを削除しているが、今後js側で必要になるようならデータヘッダーも残す
			if ( $list['name'] === $version_id . '-merge-c.php' ) {
				$merge_c_str = $this->wrap_get_contents( $heatmap_view_work_dir . $list['name'] );
				$merge_c_ary = $this->convert_tsv_to_array( $merge_c_str );

				// ここでバージョンの差異を吸収

				// ヘッダー情報は不要なため削除。必要な時が来れば別ファイルに出力すればリスクも低そう
				unset( $merge_c_ary[ self::DATA_COLUMN_HEADER ] );
				$merge_c_ary = array_values( $merge_c_ary );

				// 型変換
				foreach ( $merge_c_ary as &$merge_c ) {
					$merge_c[ self::DATA_MERGE_CLICK_1['SELECTOR_X'] ] = (int) $merge_c[ self::DATA_MERGE_CLICK_1['SELECTOR_X'] ];
					$merge_c[ self::DATA_MERGE_CLICK_1['SELECTOR_Y'] ] = (int) $merge_c[ self::DATA_MERGE_CLICK_1['SELECTOR_Y'] ];
				}

				$data['merge_c'] = $merge_c_ary;

			} elseif ( $list['name'] === $version_id . '-merge-as-v2.php' ) {
				$merge_as_str = $this->wrap_get_contents( $heatmap_view_work_dir . $list['name'] );
				$merge_as_ary = $this->convert_tsv_to_array( $merge_as_str );

				// ここでバージョンの差異を吸収

				// ヘッダー情報は不要なため削除。必要な時が来れば別ファイルに出力すればリスクも低そう
				unset( $merge_as_ary[ self::DATA_COLUMN_HEADER ] );
				$merge_as_ary = array_values( $merge_as_ary );

				// 型変換
				foreach ( $merge_as_ary as &$merge_as ) {
					$merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_HEIGHT'] ] = (int) $merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_HEIGHT'] ];
					$merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_TIME'] ]   = (float) $merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_TIME'] ];
					$merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_NUM'] ]    = (int) $merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_NUM'] ];
					$merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_2['EXIT_NUM'] ]    = (int) $merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_2['EXIT_NUM'] ];
				}
				$data['merge_as_v2'] = $merge_as_ary;

			} elseif ( $list['name'] === $version_id . '-merge-as-v1.php' || $list['name'] === $version_id . '-merge-as-v1.php' ) {
				$merge_as_str = $this->wrap_get_contents( $heatmap_view_work_dir . $list['name'] );
				$merge_as_ary = $this->convert_tsv_to_array( $merge_as_str );

				// ここでバージョンの差異を吸収

				// ヘッダー情報は不要なため削除。必要な時が来れば別ファイルに出力すればリスクも低そう
				unset( $merge_as_ary[ self::DATA_COLUMN_HEADER ] );
				$merge_as_ary = array_values( $merge_as_ary );

				// 型変換
				foreach ( $merge_as_ary as &$merge_as ) {
					$merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_1['PERCENT'] ]   = (int) $merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_1['PERCENT'] ];
					$merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_TIME'] ] = (float) $merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_TIME'] ];
					$merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_NUM'] ]  = (int) $merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_NUM'] ];
					$merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_1['EXIT_NUM'] ]  = (int) $merge_as[ self::DATA_MERGE_ATTENTION_SCROLL_1['EXIT_NUM'] ];
				}
				$data['merge_as_v1'] = $merge_as_ary;

			} elseif ( $data['merge_c'] && $data['merge_as'] ) {
				break;
			}
		}

		$data['DATA_HEATMAP_SELECTOR_NAME'] = self::DATA_MERGE_CLICK_1['SELECTOR_NAME'];
		$data['DATA_HEATMAP_SELECTOR_X']    = self::DATA_MERGE_CLICK_1['SELECTOR_X'];
		$data['DATA_HEATMAP_SELECTOR_Y']    = self::DATA_MERGE_CLICK_1['SELECTOR_Y'];

		$data['DATA_ATTENTION_SCROLL_PERCENT_V1']   = self::DATA_MERGE_ATTENTION_SCROLL_1['PERCENT'];
		$data['DATA_ATTENTION_SCROLL_STAY_TIME_V1'] = self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_TIME'];
		$data['DATA_ATTENTION_SCROLL_STAY_NUM_V1']  = self::DATA_MERGE_ATTENTION_SCROLL_1['STAY_NUM'];
		$data['DATA_ATTENTION_SCROLL_EXIT_NUM_V1']  = self::DATA_MERGE_ATTENTION_SCROLL_1['EXIT_NUM'];
		
		$data['DATA_ATTENTION_SCROLL_STAY_HEIGHT_V2'] = self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_HEIGHT'];
		$data['DATA_ATTENTION_SCROLL_STAY_TIME_V2']   = self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_TIME'];
		$data['DATA_ATTENTION_SCROLL_STAY_NUM_V2']    = self::DATA_MERGE_ATTENTION_SCROLL_2['STAY_NUM'];
		$data['DATA_ATTENTION_SCROLL_EXIT_NUM_V2']    = self::DATA_MERGE_ATTENTION_SCROLL_2['EXIT_NUM'];

		echo wp_json_encode( $data );
		die();
	}

	// 計測ボタンが押された時の挙動
	public function ajax_change_rec_checkbox() {
		global $qahm_recterm;

		$post         = array();
		$post['type'] = $this->wrap_filter_input( INPUT_POST, 'type' );
		$post['id']   = $this->wrap_filter_input( INPUT_POST, 'id' );
		$post['rec']  = $this->wrap_filter_input( INPUT_POST, 'rec' );

		// 計測有効化
		if ( $post['rec'] == 1 ) {
			if ( $qahm_recterm->exist_record( $post['type'], $post['id'] ) ) {
				$qahm_recterm->set_field( $post['type'], $post['id'], 'rec_flag', $post['rec'] );
				$qahm_recterm->set_field( $post['type'], $post['id'], 'update_date', date_i18n( 'Y-m-d' ) );
			} else {
				$qahm_recterm->set_record( $post['type'], $post['id'], true );
			}
		}
		// 計測無効化
		else {
			$qahm_recterm->set_field( $post['type'], $post['id'], 'rec_flag', $post['rec'] );
		}

		echo 'done';
		die();
	}
}
